"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut

Please see AUTHORS for contributors.
https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""


import pandas as pd
import numpy as np
from pathlib import Path
import cv2
import os
import pickle
import glob
import matplotlib.pyplot as plt
import matplotlib.colors as mcolors
from deeplabcut.utils import auxiliaryfunctions
from deeplabcut.utils import auxiliaryfunctions_3d
from matplotlib.axes._axes import _log as matplotlib_axes_logger
matplotlib_axes_logger.setLevel('ERROR')


def calibrate_cameras(config,cbrow = 8,cbcol = 6,calibrate=False,alpha=0.4):
    """This function extracts the corners points from the calibration images, calibrates the camera and stores the calibration files in the project folder (defined in the config file).
    
    Make sure you have around 20-60 pairs of calibration images. The function should be used iteratively to select the right set of calibration images. 
    
    A pair of calibration image is considered "correct", if the corners are detected correctly in both the images. It may happen that during the first run of this function, 
    the extracted corners are incorrect or the order of detected corners does not align for the corresponding views (i.e. camera-1 and camera-2 images).
    
    In such a case, remove those pairs of images and re-run this function. Once the right number of calibration images are selected, 
    use the parameter ``calibrate=True`` to calibrate the cameras.

    Parameters
    ----------
    config : string
        Full path of the config.yaml file as a string.

    cbrow : int
        Integer specifying the number of rows in the calibration image.
    
    cbcol : int
        Integer specifying the number of columns in the calibration image.

    calibrate : bool
        If this is set to True, the cameras are calibrated with the current set of calibration images. The default is ``False``
        Set it to True, only after checking the results of the corner detection method and removing dysfunctional images!
        
    alpha: float
        Floating point number between 0 and 1 specifying the free scaling parameter. When alpha = 0, the rectified images with only valid pixels are stored 
        i.e. the rectified images are zoomed in. When alpha = 1, all the pixels from the original images are retained. 
        For more details: https://docs.opencv.org/2.4/modules/calib3d/doc/camera_calibration_and_3d_reconstruction.html
        
    Example
    --------
    Linux/MacOs/Windows
    >>> deeplabcut.calibrate_camera(config)

    Once the right set of calibration images are selected, 
    >>> deeplabcut.calibrate_camera(config,calibrate=True)

    """
    # Termination criteria
    criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 30, 0.001)
    
    # Prepare object points, like (0,0,0), (1,0,0), (2,0,0) ....,(6,5,0)
    objp = np.zeros((cbrow * cbcol, 3), np.float32)
    objp[:, :2] = np.mgrid[0:cbcol, 0:cbrow].T.reshape(-1, 2)
    
    # Read the config file
    cfg_3d = auxiliaryfunctions.read_config(config)
    img_path,path_corners,path_camera_matrix,path_undistort=auxiliaryfunctions_3d.Foldernames3Dproject(cfg_3d)
    
    images = glob.glob(os.path.join(img_path,'*.jpg'))
    cam_names = cfg_3d['camera_names']
    
    # update the variable snapshot* in config file according to the name of the cameras
    try:
        for i in range(len(cam_names)):
            cfg_3d[str('config_file_'+cam_names[i])] = cfg_3d.pop(str('config_file_camera-'+str(i+1)))
        for i in range(len(cam_names)):
            cfg_3d[str('shuffle_'+cam_names[i])] = cfg_3d.pop(str('shuffle_camera-'+str(i+1)))
    except:
        pass
    
    project_path = cfg_3d['project_path']
    projconfigfile=os.path.join(str(project_path),'config.yaml')
    auxiliaryfunctions.write_config_3d(projconfigfile,cfg_3d)

    # Initialize the dictionary 
    img_shape = {}
    objpoints = {} # 3d point in real world space
    imgpoints = {} # 2d points in image plane.
    dist_pickle = {}
    stereo_params= {}
    for cam in cam_names:
        objpoints.setdefault(cam, [])
        imgpoints.setdefault(cam, [])
        dist_pickle.setdefault(cam, [])

    # Sort the images.
    images.sort(key=lambda f: int(''.join(filter(str.isdigit, f))))
    if len(images)==0:
        raise Exception("No calibration images found. Make sure the calibration images are saved as .jpg and with prefix as the camera name as specified in the config.yaml file.")
    
    for fname in images:
        for cam in cam_names:
            if cam in fname:
                filename = Path(fname).stem
                img = cv2.imread(fname)
                gray = cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)

                # Find the chess board corners
                ret, corners = cv2.findChessboardCorners(gray, (cbcol,cbrow),None,) #  (8,6) pattern (dimensions = common points of black squares)
                # If found, add object points, image points (after refining them)
                if ret == True:
                    img_shape[cam] = gray.shape[::-1]
                    objpoints[cam].append(objp)
                    corners = cv2.cornerSubPix(gray,corners,(11,11),(-1,-1),criteria)
                    imgpoints[cam].append(corners)
                    # Draw the corners and store the images
                    img = cv2.drawChessboardCorners(img, (cbcol,cbrow), corners,ret)
                    cv2.imwrite(os.path.join(str(path_corners),filename+'_corner.jpg'),img)
                else:
                    print("Corners not found for the image %s" %Path(fname).name)
    try:
        h,  w = img.shape[:2]
    except:
        raise Exception("It seems that the name of calibration images does not match with the camera names in the config file. Please make sure that the calibration images are named with camera names as specified in the config.yaml file.")

    # Perform calibration for each cameras and store the matrices as a pickle file
    if calibrate == True:
        # Calibrating each camera
        for cam in cam_names:
            ret, mtx, dist, rvecs, tvecs = cv2.calibrateCamera(objpoints[cam], imgpoints[cam], img_shape[cam],None,None)

            # Save the camera calibration result for later use (we won't use rvecs / tvecs)
            dist_pickle[cam] = {'mtx':mtx , 'dist':dist, 'objpoints':objpoints[cam] ,'imgpoints':imgpoints[cam] }
            pickle.dump( dist_pickle, open( os.path.join(path_camera_matrix,cam+'_intrinsic_params.pickle'), "wb" ) )
            print('Saving intrinsic camera calibration matrices for %s as a pickle file in %s'%(cam, os.path.join(path_camera_matrix)))
            
            # Compute mean re-projection errors for individual cameras
            mean_error = 0
            for i in range(len(objpoints[cam])):
                imgpoints_proj, _ = cv2.projectPoints(objpoints[cam][i], rvecs[i], tvecs[i], mtx, dist)
                error = cv2.norm(imgpoints[cam][i],imgpoints_proj, cv2.NORM_L2)/len(imgpoints_proj)
                mean_error += error
            print("Mean re-projection error for %s images: %.3f pixels " %(cam, mean_error/len(objpoints[cam])))

        # Compute stereo calibration for each pair of cameras
        camera_pair = [[cam_names[0], cam_names[1]]]
        for pair in camera_pair:
            print("Computing stereo calibration for " %pair)
            retval, cameraMatrix1, distCoeffs1, cameraMatrix2, distCoeffs2, R, T, E, F = cv2.stereoCalibrate(objpoints[pair[0]],imgpoints[pair[0]],imgpoints[pair[1]],dist_pickle[pair[0]]['mtx'],dist_pickle[pair[0]]['dist'], dist_pickle[pair[1]]['mtx'], dist_pickle[pair[1]]['dist'],(h,  w),flags = cv2.CALIB_FIX_INTRINSIC)

            # Stereo Rectification
            rectify_scale = alpha # Free scaling parameter check this https://docs.opencv.org/2.4/modules/calib3d/doc/camera_calibration_and_3d_reconstruction.html#fisheye-stereorectify
            R1, R2, P1, P2, Q, roi1, roi2 = cv2.stereoRectify(cameraMatrix1, distCoeffs1, cameraMatrix2, distCoeffs2, (h, w), R, T, alpha = rectify_scale)
            
            stereo_params[pair[0]+'-'+pair[1]] = {"cameraMatrix1": cameraMatrix1,"cameraMatrix2": cameraMatrix2,"distCoeffs1": distCoeffs1,"distCoeffs2": distCoeffs2,"R":R,"T":T,"E":E,"F":F,
                         "R1":R1,
                         "R2":R2,
                         "P1":P1,
                         "P2":P2,
                         "roi1":roi1,
                         "roi2":roi2,
                         "Q":Q,
                         "image_shape":[img_shape[pair[0]],img_shape[pair[1]]]}
            
        print('Saving the stereo parameters for every pair of cameras as a pickle file in %s'%str(os.path.join(path_camera_matrix)))
        
        auxiliaryfunctions.write_pickle(os.path.join(path_camera_matrix,'stereo_params.pickle'),stereo_params)
        print("Camera calibration done! Use the function ``check_undistortion`` to check the check the calibration")
    else:
        print("Corners extracted! You may check for the extracted corners in the directory %s and remove the pair of images where the corners are incorrectly detected. If all the corners are detected correctly with right order, then re-run the same function and use the flag ``calibrate=True``, to calbrate the camera."%str(path_corners))



def check_undistortion(config,cbrow = 8,cbcol = 6,plot=True):
    """
    This function undistorts the calibration images based on the camera matrices and stores them in the project folder(defined in the config file) 
    to visually check if the camera matrices are correct. 

    Parameters
    ----------
    config : string
        Full path of the config.yaml file as a string.

    cbrow : int
        Int specifying the number of rows in the calibration image.
    
    cbcol : int
        Int specifying the number of columns in the calibration image.

    plot : bool
        If this is set to True, the results of undistortion are saved as plots. The default is ``True``; if provided it must be either ``True`` or ``False``.

    Example
    --------
    Linux/MacOs/Windows
    >>> deeplabcut.check_undistortion(config, cbrow = 8,cbcol = 6)

    """
    
    # Read the config file
    criteria = (cv2.TERM_CRITERIA_EPS + cv2.TERM_CRITERIA_MAX_ITER, 30, 0.001)
    cfg_3d = auxiliaryfunctions.read_config(config)
    img_path,path_corners,path_camera_matrix,path_undistort=auxiliaryfunctions_3d.Foldernames3Dproject(cfg_3d)
    
    #colormap = plt.get_cmap(cfg_3d['colormap'])
    markerSize = cfg_3d['dotsize']
    alphaValue = cfg_3d['alphaValue']
    markerType = cfg_3d['markerType']
    markerColor = cfg_3d['markerColor']
    cam_names = cfg_3d['camera_names']
    
    images = glob.glob(os.path.join(img_path,'*.jpg'))

    # Sort the images
    images.sort(key=lambda f: int(''.join(filter(str.isdigit, f))))
    '''
    for fname in images:
        for cam in cam_names:
            if cam in fname:
                filename = Path(fname).stem
                ext = Path(fname).suffix
                img = cv2.imread(fname)
                gray = cv2.cvtColor(img,cv2.COLOR_BGR2GRAY)
    '''
    camera_pair = [[cam_names[0], cam_names[1]]]
    stereo_params = auxiliaryfunctions.read_pickle(os.path.join(path_camera_matrix,'stereo_params.pickle'))
    
    for pair in camera_pair:
        map1_x,map1_y = cv2.initUndistortRectifyMap(stereo_params[pair[0]+'-'+pair[1]]["cameraMatrix1"], stereo_params[pair[0]+'-'+pair[1]]["distCoeffs1"], stereo_params[pair[0]+'-'+pair[1]]["R1"], stereo_params[pair[0]+'-'+pair[1]]["P1"],(stereo_params[pair[0]+'-'+pair[1]]["image_shape"][0]), cv2.CV_16SC2)
        map2_x,map2_y = cv2.initUndistortRectifyMap(stereo_params[pair[0]+'-'+pair[1]]["cameraMatrix2"], stereo_params[pair[0]+'-'+pair[1]]["distCoeffs2"], stereo_params[pair[0]+'-'+pair[1]]["R2"], stereo_params[pair[0]+'-'+pair[1]]["P2"],(stereo_params[pair[0]+'-'+pair[1]]["image_shape"][1]), cv2.CV_16SC2)
        cam1_undistort = []
        cam2_undistort = []
        
        for fname in images:
            if pair[0] in fname:
                filename = Path(fname).stem
                img1 = cv2.imread(fname)
                gray1 = cv2.cvtColor(img1,cv2.COLOR_BGR2GRAY)
                h,  w = img1.shape[:2]
                _, corners1 = cv2.findChessboardCorners(gray1, (cbcol,cbrow),None,)
                corners_origin1 = cv2.cornerSubPix(gray1,corners1,(11,11),(-1,-1),criteria)

                # Remapping dataFrame_camera1_undistort
                im_remapped1 = cv2.remap(img1, map1_x, map1_y, cv2.INTER_LANCZOS4)
                imgpoints_proj_undistort = cv2.undistortPoints(src=corners_origin1, cameraMatrix =stereo_params[pair[0]+'-'+pair[1]]["cameraMatrix1"], distCoeffs = stereo_params[pair[0]+'-'+pair[1]]["distCoeffs1"],P=stereo_params[pair[0]+'-'+pair[1]]["P1"],R=stereo_params[pair[0]+'-'+pair[1]]["R1"])
                cam1_undistort.append(imgpoints_proj_undistort)
                cv2.imwrite(os.path.join(str(path_undistort),filename+'_undistort.jpg'),im_remapped1)
                imgpoints_proj_undistort = []
                
            elif pair[1] in fname:
                filename = Path(fname).stem
                img2 = cv2.imread(fname)
                gray2 = cv2.cvtColor(img2,cv2.COLOR_BGR2GRAY)
                h,  w = img2.shape[:2]
                _, corners2 = cv2.findChessboardCorners(gray2, (cbcol,cbrow),None,)
                corners_origin2 = cv2.cornerSubPix(gray2,corners2,(11,11),(-1,-1),criteria)

                # Remapping
                im_remapped2 = cv2.remap(img2, map2_x, map2_y, cv2.INTER_LANCZOS4)
                imgpoints_proj_undistort2 = cv2.undistortPoints(src=corners_origin2, cameraMatrix =stereo_params[pair[0]+'-'+pair[1]]["cameraMatrix2"], distCoeffs = stereo_params[pair[0]+'-'+pair[1]]["distCoeffs2"],P=stereo_params[pair[0]+'-'+pair[1]]["P2"],R=stereo_params[pair[0]+'-'+pair[1]]["R2"])
                cam2_undistort.append(imgpoints_proj_undistort2)
                cv2.imwrite(os.path.join(str(path_undistort),filename+'_undistort.jpg'),im_remapped2)
                imgpoints_proj_undistort2 = []
        
        cam1_undistort = np.array(cam1_undistort)
        cam2_undistort = np.array(cam2_undistort)
        print("All images are undistorted and stored in %s"%str(path_undistort))
        print("Use the function ``triangulate`` to undistort the dataframes and compute the triangulation")
        
        if plot == True:
            f1, (ax1, ax2) = plt.subplots(1, 2, figsize=(20,10))
            f1.suptitle(str("Original Image: Views from "+ pair[0] + " and " + pair[1]), fontsize=25)

            # Display images in RGB
            ax1.imshow(cv2.cvtColor(img1, cv2.COLOR_BGR2RGB))
            ax2.imshow(cv2.cvtColor(img2, cv2.COLOR_BGR2RGB))

            norm = mcolors.Normalize(vmin=0.0, vmax=cam1_undistort.shape[1])
            plt.savefig(os.path.join(str(path_undistort),"Original_Image.png"))

            # Plot the undistorted corner points 
            f2, (ax1, ax2) = plt.subplots(1, 2, figsize=(20,10))
            f2.suptitle("Undistorted corner points on camera-1 and camera-2", fontsize=25)
            ax1.imshow(cv2.cvtColor(im_remapped1, cv2.COLOR_BGR2RGB))
            ax2.imshow(cv2.cvtColor(im_remapped2, cv2.COLOR_BGR2RGB))
            for i in range(0,cam1_undistort.shape[1]):
                ax1.scatter([cam1_undistort[-1][i,0,0]], [cam1_undistort[-1][i,0,1]], marker=markerType, s=markerSize,color=markerColor,alpha = alphaValue)
                ax2.scatter([cam2_undistort[-1][i,0,0]], [cam2_undistort[-1][i,0,1]], marker=markerType, s=markerSize,color=markerColor,alpha = alphaValue)
            plt.savefig(os.path.join(str(path_undistort),"undistorted_points.png"))

            # Triangulate
            triangulate = auxiliaryfunctions_3d.compute_triangulation_calibration_images(stereo_params[pair[0]+'-'+pair[1]], cam1_undistort, cam2_undistort,path_undistort,cfg_3d,plot= True)
            auxiliaryfunctions.write_pickle('triangulate.pickle',triangulate)



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut

Please see AUTHORS for contributors.
https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""

import os
from pathlib import Path
import deeplabcut
from deeplabcut.utils import auxiliaryfunctions

def load_demo_data(config,createtrainingset=True):
  """
  Loads the demo data. Make sure that you are in the same directory where you have downloaded or cloned the deeplabcut.

  Parameter
    ----------
    config : string
        Full path of the config.yaml file of the provided demo dataset as a string.

    Example
    --------
    >>> deeplabcut.load_demo_data('config.yaml')
    --------
  """
  config = Path(config).resolve()
  config = str(config)
  
  transform_data(config)
  if createtrainingset:
      print("Loaded, now creating training data...")
      deeplabcut.create_training_dataset(config, num_shuffles=1)

def transform_data(config):
    """
    This function adds the full path to labeling dataset.
    It also adds the correct path to the video file in the config file.
    """
    import pandas as pd

    cfg = auxiliaryfunctions.read_config(config)
    project_path = str(Path(config).parents[0])

    cfg['project_path'] = project_path
    if 'Reaching' in project_path:
        video_file = os.path.join(project_path, 'videos','reachingvideo1.avi')
    elif 'openfield' in project_path:
        video_file = os.path.join(project_path, 'videos','m4s1.mp4')
    else:
        print("This is not an offical demo dataset.")
    
    if 'WILL BE AUTOMATICALLY UPDATED BY DEMO CODE' in cfg['video_sets'].keys():
        cfg['video_sets'][str(video_file)] = cfg['video_sets'].pop('WILL BE AUTOMATICALLY UPDATED BY DEMO CODE')

    auxiliaryfunctions.write_config(config,cfg)



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut
Please see AUTHORS for contributors.

https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0

"""

import wx
import os,sys,pydoc
import deeplabcut
from deeplabcut.utils import auxiliaryfunctions
media_path = os.path.join(deeplabcut.__path__[0], 'gui' , 'media')
logo = os.path.join(media_path,'logo.png')

class Analyze_videos(wx.Panel):
    """
    """

    def __init__(self, parent,gui_size,cfg):
        """Constructor"""
        wx.Panel.__init__(self, parent=parent)
        # variable initilization
        self.filelist = []
        self.bodyparts = []
        self.config = cfg
        self.draw = False
        # design the panel
        self.sizer = wx.GridBagSizer(5, 10)

        text = wx.StaticText(self, label="DeepLabCut - Step 7. Analyze videos")
        self.sizer.Add(text, pos=(0, 0), flag=wx.TOP|wx.LEFT|wx.BOTTOM,border=15)
        # Add logo of DLC
        icon = wx.StaticBitmap(self, bitmap=wx.Bitmap(logo))
        self.sizer.Add(icon, pos=(0, 9), flag=wx.TOP|wx.RIGHT|wx.ALIGN_RIGHT,border=5)

        line1 = wx.StaticLine(self)
        self.sizer.Add(line1, pos=(1, 0), span=(1, 10),flag=wx.EXPAND|wx.BOTTOM, border=10)

        self.cfg_text = wx.StaticText(self, label="Select the config file")
        self.sizer.Add(self.cfg_text, pos=(2, 0), flag=wx.TOP|wx.LEFT, border=10)

        if sys.platform=='darwin':
            self.sel_config = wx.FilePickerCtrl(self, path="",style=wx.FLP_USE_TEXTCTRL,message="Choose the config.yaml file", wildcard="*.yaml")
        else:
            self.sel_config = wx.FilePickerCtrl(self, path="",style=wx.FLP_USE_TEXTCTRL,message="Choose the config.yaml file", wildcard="config.yaml")

        self.sizer.Add(self.sel_config, pos=(2, 1),span=(1,3),flag=wx.TOP|wx.EXPAND, border=5)
        self.sel_config.SetPath(self.config)
        self.sel_config.Bind(wx.EVT_FILEPICKER_CHANGED, self.select_config)

        self.vids = wx.StaticText(self, label="Choose the videos")
        self.sizer.Add(self.vids, pos=(3, 0), flag=wx.TOP|wx.LEFT, border=10)

        self.sel_vids = wx.Button(self, label="Select videos to analyze")
        self.sizer.Add(self.sel_vids, pos=(3, 1), flag=wx.TOP|wx.EXPAND, border=5)
        self.sel_vids.Bind(wx.EVT_BUTTON, self.select_videos)

        sb = wx.StaticBox(self, label="Additional Attributes")
        boxsizer = wx.StaticBoxSizer(sb, wx.VERTICAL)

        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        hbox3 = wx.BoxSizer(wx.HORIZONTAL)
        hbox4 = wx.BoxSizer(wx.HORIZONTAL)


#        self.shuffles = wx.SpinCtrl(self, value='1',min=1,max=100)
#        shuffles_text_boxsizer.Add(self.shuffles,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        videotype_text = wx.StaticBox(self, label="Specify the videotype")
        videotype_text_boxsizer = wx.StaticBoxSizer(videotype_text, wx.VERTICAL)

        videotypes = ['.avi', '.mp4', '.mov']
        self.videotype = wx.ComboBox(self,choices = videotypes,style = wx.CB_READONLY)
        self.videotype.SetValue('.avi')
        videotype_text_boxsizer.Add(self.videotype,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        shuffle_text = wx.StaticBox(self, label="Specify the shuffle")
        shuffle_boxsizer = wx.StaticBoxSizer(shuffle_text, wx.VERTICAL)
        self.shuffle = wx.SpinCtrl(self, value='1',min=1,max=100)
        shuffle_boxsizer.Add(self.shuffle,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        trainingset = wx.StaticBox(self, label="Specify the trainingset index")
        trainingset_boxsizer = wx.StaticBoxSizer(trainingset, wx.VERTICAL)
        self.trainingset = wx.SpinCtrl(self, value='0',min=0,max=100)
        trainingset_boxsizer.Add(self.trainingset,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        destfolder_text = wx.StaticBox(self, label="Specify destination folder")
        destfolderboxsizer = wx.StaticBoxSizer(destfolder_text, wx.VERTICAL)
        self.sel_destfolder = wx.DirPickerCtrl(self, path="",style=wx.FLP_USE_TEXTCTRL,message="Choose the destination folder")
        self.sel_destfolder.SetPath("None")
        self.destfolder = None
        self.sel_destfolder.Bind(wx.EVT_FILEPICKER_CHANGED, self.select_destfolder)
        destfolderboxsizer.Add(self.sel_destfolder,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        hbox1.Add(videotype_text_boxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox1.Add(shuffle_boxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox1.Add(trainingset_boxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox1.Add(destfolderboxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)

        boxsizer.Add(hbox1,0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)


        self.csv = wx.RadioBox(self, label='Want to save result(s) as csv?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.csv.SetSelection(1)

        self.cropping = wx.RadioBox(self, label='Want to crop the videos?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.cropping.SetSelection(1)

        self.dynamic = wx.RadioBox(self, label='Want to dynamically crop bodyparts?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.dynamic.SetSelection(1)

        self.filter = wx.RadioBox(self, label='Want to filter the predictions?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.filter.SetSelection(1)

        self.trajectory = wx.RadioBox(self, label='Want to plot the trajectories?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.trajectory.Bind(wx.EVT_RADIOBOX,self.chooseOption)
        self.trajectory.SetSelection(1)

        config_file = auxiliaryfunctions.read_config(self.config)
        bodyparts = config_file['bodyparts']
        self.trajectory_to_plot = wx.CheckListBox(self, choices=bodyparts, style=0,name = "Select the bodyparts")
        self.trajectory_to_plot.Bind(wx.EVT_CHECKLISTBOX,self.getbp)
        self.trajectory_to_plot.SetCheckedItems(range(len(bodyparts)))
        self.trajectory_to_plot.Hide()

        self.create_labeled_videos = wx.RadioBox(self, label='Create labeled video(s)? (see next tab for more options)', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.create_labeled_videos.Bind(wx.EVT_RADIOBOX, self.choose_create_labeled_video_options)
        self.create_labeled_videos.SetSelection(1)

        self.draw_skeleton = wx.RadioBox(self, label='Include the skeleton in the video?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.draw_skeleton.Bind(wx.EVT_RADIOBOX, self.choose_draw_skeleton_options)
        self.draw_skeleton.SetSelection(1)
        self.draw_skeleton.Hide()

        self.trail_points_text = wx.StaticBox(self, label="Specify the number of trail points")
        trail_pointsboxsizer = wx.StaticBoxSizer(self.trail_points_text, wx.VERTICAL)
        self.trail_points = wx.SpinCtrl(self, value='1')
        trail_pointsboxsizer.Add(self.trail_points,20, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        self.trail_points_text.Hide()
        self.trail_points.Hide()

        hbox2.Add(self.csv,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox2.Add(self.filter,10,wx.EXPAND|wx.TOP|wx.BOTTOM,5)
        hbox2.Add(self.trajectory,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox2.Add(self.trajectory_to_plot,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        boxsizer.Add(hbox2,0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        hbox3.Add(self.cropping,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox3.Add(self.dynamic,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox3.Add(self.create_labeled_videos,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        boxsizer.Add(hbox3,0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        hbox4.Add(self.draw_skeleton,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox4.Add(trail_pointsboxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        boxsizer.Add(hbox4,0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        self.sizer.Add(boxsizer, pos=(4, 0), span=(1, 10),flag=wx.EXPAND|wx.TOP|wx.LEFT|wx.RIGHT , border=10)



        self.help_button = wx.Button(self, label='Help')
        self.sizer.Add(self.help_button, pos=(5, 0), flag=wx.LEFT, border=10)
        self.help_button.Bind(wx.EVT_BUTTON, self.help_function)

        self.ok = wx.Button(self, label="RUN")
        self.sizer.Add(self.ok, pos=(5, 8), flag=wx.BOTTOM|wx.RIGHT, border=10)
        self.ok.Bind(wx.EVT_BUTTON, self.analyze_videos)

        self.reset = wx.Button(self, label="Reset")
        self.sizer.Add(self.reset, pos=(5, 1), span=(1, 1),flag=wx.BOTTOM|wx.RIGHT, border=10)
        self.reset.Bind(wx.EVT_BUTTON, self.reset_analyze_videos)

        self.sizer.AddGrowableCol(2)

        self.SetSizer(self.sizer)
        self.sizer.Fit(self)

    def help_function(self,event):

        filepath= 'help.txt'
        f = open(filepath, 'w')
        sys.stdout = f
        fnc_name = 'deeplabcut.analyze_videos'
        pydoc.help(fnc_name)
        f.close()
        sys.stdout = sys.__stdout__
        help_file = open("help.txt","r+")
        help_text = help_file.read()
        wx.MessageBox(help_text,'Help',wx.OK | wx.ICON_INFORMATION)
        os.remove('help.txt')

    def select_destfolder(self,event):
        self.destfolder = self.sel_destfolder.GetPath()

    def select_config(self,event):
        """
        """
        self.config = self.sel_config.GetPath()

    def select_videos(self,event):
        """
        Selects the videos from the directory
        """
        cwd = os.getcwd()
        dlg = wx.FileDialog(self, "Select videos to analyze", cwd, "", "*.*", wx.FD_MULTIPLE)
        if dlg.ShowModal() == wx.ID_OK:
            self.vids = dlg.GetPaths()
            self.filelist = self.filelist + self.vids
            self.sel_vids.SetLabel("Total %s Videos selected" %len(self.filelist))

    def choose_draw_skeleton_options(self,event):
        # if self.draw_skeleton.IsShow():
        if self.draw_skeleton.GetStringSelection() == "Yes":
            self.draw = True
        else:
            self.draw = False

    def choose_create_labeled_video_options(self,event):
        if self.create_labeled_videos.GetStringSelection() == "Yes":
            self.draw_skeleton.Show()
            self.trail_points_text.Show()
            self.trail_points.Show()
            self.SetSizer(self.sizer)
            self.sizer.Fit(self)
        else:
            self.draw_skeleton.Hide()
            self.trail_points_text.Hide()
            self.trail_points.Hide()
            self.SetSizer(self.sizer)
            self.sizer.Fit(self)

    def analyze_videos(self,event):

        shuffle = self.shuffle.GetValue()
        trainingsetindex = self.trainingset.GetValue()

        if self.csv.GetStringSelection() == "Yes":
            save_as_csv = True
        else:
            save_as_csv = False

        if self.cropping.GetStringSelection() == "No":
            cropping = None
        else:
            cropping = True

        if self.dynamic.GetStringSelection() == "No":
            dynamic = (False, .5, 10)
        else:
            dynamic = (True, .5, 10)

        if self.filter.GetStringSelection() == "No":
            filter = None
        else:
            filter = True

        # if self.draw_skeleton.GetStringSelection() == "Yes":
        #     self.draw = True
        # else:
        #     self.draw = True

#        print(self.config,self.filelist,self.videotype.GetValue(),shuffle,trainingsetindex,gputouse=None,save_as_csv=save_as_csv,destfolder=self.destfolder,cropping=cropping)
        deeplabcut.analyze_videos(self.config,self.filelist,videotype=self.videotype.GetValue(),shuffle=shuffle,trainingsetindex=trainingsetindex,gputouse=None,save_as_csv=save_as_csv,destfolder=self.destfolder,cropping=cropping, dynamic=dynamic)
        if self.filter.GetStringSelection() == "Yes":
            deeplabcut.filterpredictions(self.config, self.filelist, videotype=self.videotype.GetValue(), shuffle=shuffle, trainingsetindex=trainingsetindex, filtertype='median', windowlength=5, p_bound=0.001, ARdegree=3, MAdegree=1, alpha=0.01, save_as_csv=True, destfolder=self.destfolder)

        if self.create_labeled_videos.GetStringSelection() == "Yes":
            deeplabcut.create_labeled_video(self.config,self.filelist,self.videotype.GetValue(),shuffle=shuffle, trainingsetindex=trainingsetindex, draw_skeleton= self.draw,trailpoints = self.trail_points.GetValue(), filtered=True)

        if self.trajectory.GetStringSelection() == "Yes":
            deeplabcut.plot_trajectories(self.config, self.filelist, displayedbodyparts=self.bodyparts,
                                         videotype=self.videotype.GetValue(), shuffle=shuffle, trainingsetindex=trainingsetindex, filtered=True, showfigures=False, destfolder=self.destfolder)

    def reset_analyze_videos(self,event):
        """
        Reset to default
        """
        self.config = []
        self.sel_config.SetPath("")
        self.videotype.SetStringSelection(".avi")
        self.sel_vids.SetLabel("Select videos to analyze")
        self.filelist = []
#        self.gpu.SetValue(-1)
        self.shuffle.SetValue(1)
        self.trainingset.SetValue(0)
        self.csv.SetSelection(1)
        self.filter.SetSelection(1)
        self.trajectory.SetSelection(1)
        self.cropping.SetSelection(1)
        self.dynamic.SetSelection(1)
        self.create_labeled_videos.SetSelection(1)
        self.sel_destfolder.SetPath("None")
        if self.draw_skeleton.IsShown():
            self.draw_skeleton.SetSelection(1)
            self.draw_skeleton.Hide()
            self.trail_points_text.Hide()
            self.trail_points.Hide()
            self.SetSizer(self.sizer)
            self.sizer.Fit(self)

    def chooseOption(self,event):
        if self.trajectory.GetStringSelection() == 'Yes':
            self.trajectory_to_plot.Show()
            self.getbp(event)
            self.SetSizer(self.sizer)
            self.sizer.Fit(self) #this sets location.
        if self.trajectory.GetStringSelection() == 'No':
            self.trajectory_to_plot.Hide()
            self.SetSizer(self.sizer)
            self.sizer.Fit(self)
            self.bodyparts = []

    def getbp(self,event):
        self.bodyparts = list(self.trajectory_to_plot.GetCheckedStrings())



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut

Please see AUTHORS for contributors.
https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""


def extract_frames(config,mode='automatic',algo='kmeans',crop=False,userfeedback=True,cluster_step=1,cluster_resizewidth=30,cluster_color=False,opencv=True,slider_width=25):
    """
    Extracts frames from the videos in the config.yaml file. Only the videos in the config.yaml will be used to select the frames.\n
    Use the function ``add_new_video`` at any stage of the project to add new videos to the config file and extract their frames.
    
    The provided function either selects frames from the videos in a randomly and temporally uniformly distributed way (uniform), \n 
    by clustering based on visual appearance (k-means), or by manual selection. 
    
    Three important parameters for automatic extraction: numframes2pick, start and stop are set in the config file. 
    
    Please refer to the user guide for more details on methods and parameters https://www.biorxiv.org/content/biorxiv/early/2018/11/24/476531.full.pdf
    
    Parameters
    ----------
    config : string
        Full path of the config.yaml file as a string.
        
    mode : string
        String containing the mode of extraction. It must be either ``automatic`` or ``manual``.
        
    algo : string 
        String specifying the algorithm to use for selecting the frames. Currently, deeplabcut supports either ``kmeans`` or ``uniform`` based selection. This flag is
        only required for ``automatic`` mode and the default is ``uniform``. For uniform, frames are picked in temporally uniform way, kmeans performs clustering on downsampled frames (see user guide for details).
        Note: color information is discarded for kmeans, thus e.g. for camouflaged octopus clustering one might want to change this. 
        
    crop : bool, optional
        If this is set to True, a user interface pops up with a frame to select the cropping parameters. Use the left click to draw a cropping area and hit the button set cropping parameters to save the cropping parameters for a video.
        The default is ``False``; if provided it must be either ``True`` or ``False``.
            
    userfeedback: bool, optional
        If this is set to false during automatic mode then frames for all videos are extracted. The user can set this to true, which will result in a dialog,
        where the user is asked for each video if (additional/any) frames from this video should be extracted. Use this, e.g. if you have already labeled
        some folders and want to extract data for new videos. 
    
    cluster_resizewidth: number, default: 30
        For k-means one can change the width to which the images are downsampled (aspect ratio is fixed).
    
    cluster_step: number, default: 1
        By default each frame is used for clustering, but for long videos one could only use every nth frame (set by: cluster_step). This saves memory before clustering can start, however, 
        reading the individual frames takes longer due to the skipping.
    
    cluster_color: bool, default: False
        If false then each downsampled image is treated as a grayscale vector (discarding color information). If true, then the color channels are considered. This increases 
        the computational complexity. 
    
    opencv: bool, default: True
        Uses openCV for loading & extractiong (otherwise moviepy (legacy))
        
    slider_width: number, default: 25
        Width of the video frames slider, in percent of window
        
    Examples
    --------
    for selecting frames automatically with 'kmeans' and want to crop the frames.
    >>> deeplabcut.extract_frames('/analysis/project/reaching-task/config.yaml','automatic','kmeans',True)
    --------
    for selecting frames automatically with 'kmeans' and considering the color information.
    >>> deeplabcut.extract_frames('/analysis/project/reaching-task/config.yaml','automatic','kmeans',cluster_color=True)
    --------
    for selecting frames automatically with 'uniform' and want to crop the frames.
    >>> deeplabcut.extract_frames('/analysis/project/reaching-task/config.yaml','automatic',crop=True)
    --------
    for selecting frames manually,
    >>> deeplabcut.extract_frames('/analysis/project/reaching-task/config.yaml','manual')
    --------
    for selecting frames manually, with a 60% wide frames slider
    >>> deeplabcut.extract_frames('/analysis/project/reaching-task/config.yaml','manual', slider_width=60)
    
    While selecting the frames manually, you do not need to specify the ``crop`` parameter in the command. Rather, you will get a prompt in the graphic user interface to choose 
    if you need to crop or not.
    --------
    
    """
    import os
    import sys
    import numpy as np
    from pathlib import Path
    from skimage import io
    from skimage.util import img_as_ubyte
    import matplotlib.pyplot as plt
    import matplotlib.patches as patches
    from deeplabcut.utils import frameselectiontools
    from deeplabcut.utils import auxiliaryfunctions
    from matplotlib.widgets import RectangleSelector

    if mode == "manual":
        wd = Path(config).resolve().parents[0]
        os.chdir(str(wd))
        from deeplabcut.generate_training_dataset import frame_extraction_toolbox
        from deeplabcut.utils import select_crop_parameters
        frame_extraction_toolbox.show(config, slider_width)
        
    elif mode == "automatic":
        config_file = Path(config).resolve()
        cfg = auxiliaryfunctions.read_config(config_file)
        print("Config file read successfully.")
        
        numframes2pick = cfg['numframes2pick']
        start = cfg['start']
        stop = cfg['stop']
        
        # Check for variable correctness
        if start>1 or stop>1 or start<0 or stop<0 or start>=stop:
            raise Exception("Erroneous start or stop values. Please correct it in the config file.")
        if numframes2pick<1 and not int(numframes2pick):
            raise Exception("Perhaps consider extracting more, or a natural number of frames.")
        
        videos = cfg['video_sets'].keys()
        if opencv:
            import cv2
        else:
            from moviepy.editor import VideoFileClip
        for vindex,video in enumerate(videos):
            #plt.close("all")
            global coords
            coords = cfg['video_sets'][video]['crop'].split(',')
            
            if userfeedback:
                print("Do you want to extract (perhaps additional) frames for video:", video, "?")
                askuser = input("yes/no")
            else:
                askuser="yes"
                
            if askuser=='y' or askuser=='yes' or askuser=='Ja' or askuser=='ha': # multilanguage support :)
                if opencv:
                    cap=cv2.VideoCapture(video)
                    fps = cap.get(5) #https://docs.opencv.org/2.4/modules/highgui/doc/reading_and_writing_images_and_video.html#videocapture-get
                    nframes = int(cap.get(7))
                    duration=nframes*1./fps
                else:
                    #Moviepy:
                    clip = VideoFileClip(video)
                    fps=clip.fps
                    duration=clip.duration
                    nframes=int(np.ceil(clip.duration*1./fps))
                indexlength = int(np.ceil(np.log10(nframes)))
                if crop==True:
                    from deeplabcut.utils import select_crop_parameters
                    if opencv:
                        cap.set(2,start*duration)
                        ret, frame = cap.read()
                        if ret:
                            image=cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
                    else:
                        image = clip.get_frame(start*clip.duration) #frame is accessed by index *1./clip.fps (fps cancels)
                    
                    fname = Path(video)
                    output_path = Path(config).parents[0] / 'labeled-data' / fname.stem
                    
                    if output_path.exists() :
                        fig,ax = plt.subplots(1)
                        # Call the GUI to select the cropping parameters
                        coords = select_crop_parameters.show(config,image)
                        # Update the config.yaml file with current cropping parameters
                        cfg['video_sets'][video] = {'crop': ', '.join(map(str, [int(coords[0]), int(coords[1]), int(coords[2]), int(coords[3])]))}
                        auxiliaryfunctions.write_config(config_file,cfg)

                        if len(os.listdir(output_path))==0: #check if empty
                                #store full frame from random location (good for augmentation)
                                index=int(start*duration+np.random.rand()*duration*(stop-start))
                                if opencv:
                                    cap.set(1,index)
                                    ret, frame = cap.read()
                                    if ret:
                                        image=img_as_ubyte(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
                                else:
                                    image = img_as_ubyte(clip.get_frame(index * 1. / clip.fps))
                                    clip=clip.crop(y1 = int(coords[2]),y2 = int(coords[3]),x1 = int(coords[0]), x2 = int(coords[1])) #now crop clip
                                    
                                saveimg = str(output_path) +'/img'+ str(index).zfill(indexlength) + ".png"
                                io.imsave(saveimg, image)

                        else:
                              askuser=input ("The directory already contains some frames. Do you want to add to it?(yes/no): ")
                              if askuser=='y' or askuser=='yes' or askuser=='Y' or askuser=='Yes':
                                  #clip=clip.crop(y1 = int(coords[2]),y2 = int(coords[3]),x1 = int(coords[0]), x2 = int(coords[1]))
                                  index=int(start*duration+np.random.rand()*duration*(stop-start))
                                  if opencv:
                                    cap.set(1,index)
                                    ret, frame = cap.read()
                                    if ret:
                                        image=img_as_ubyte(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
                                  else:
                                      image = img_as_ubyte(clip.get_frame(index * 1. / clip.fps))
                                      clip=clip.crop(y1 = int(coords[2]),y2 = int(coords[3]),x1 = int(coords[0]), x2 = int(coords[1]))
                            
                                  saveimg = str(output_path) +'/img'+ str(index).zfill(indexlength) + ".png"
                                  io.imsave(saveimg, image)
                                  pass
                              else:
                                  sys.exit("Delete the frames and try again later!")
                    
                            
                else:
                    numframes2pick=cfg['numframes2pick']+1 # without cropping a full size frame will not be extracted >> thus one more frame should be selected in next stage.
                    
                print("Extracting frames based on %s ..." %algo)

                if algo =='uniform': #extract n-1 frames (0 was already stored)
                    if opencv:
                        frames2pick=frameselectiontools.UniformFramescv2(cap,numframes2pick-1,start,stop)
                    else:
                        frames2pick=frameselectiontools.UniformFrames(clip,numframes2pick-1,start,stop)
                elif algo =='kmeans':
                    if opencv:
                        frames2pick=frameselectiontools.KmeansbasedFrameselectioncv2(cap,numframes2pick-1,start,stop,crop,coords,step=cluster_step,resizewidth=cluster_resizewidth,color=cluster_color)
                    else:
                        frames2pick=frameselectiontools.KmeansbasedFrameselection(clip,numframes2pick-1,start,stop,step=cluster_step,resizewidth=cluster_resizewidth,color=cluster_color)
                else:
                    print("Please implement this method yourself and send us a pull request! Otherwise, choose 'uniform' or 'kmeans'.")
                    frames2pick=[]
                
                output_path = Path(config).parents[0] / 'labeled-data' / Path(video).stem
                if opencv:
                    for index in frames2pick:
                            cap.set(1,index) #extract a particular frame
                            ret, frame = cap.read()
                            if ret:
                                image=img_as_ubyte(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
                                img_name = str(output_path) +'/img'+ str(index).zfill(indexlength) + ".png"
                                if crop:
                                    io.imsave(img_name,image[int(coords[2]):int(coords[3]),int(coords[0]):int(coords[1]),:]) #y1 = int(coords[2]),y2 = int(coords[3]),x1 = int(coords[0]), x2 = int(coords[1]
                                else:
                                    io.imsave(img_name,image)
                            else:
                                print("Frame", index, " not found!")
                    cap.release()
                else:
                    for index in frames2pick:
                        try:
                            image = img_as_ubyte(clip.get_frame(index * 1. / clip.fps))
                            img_name = str(output_path) +'/img'+ str(index).zfill(indexlength) + ".png"
                            io.imsave(img_name,image)
                            if np.var(image)==0: #constant image
                                print("Seems like black/constant images are extracted from your video. Perhaps consider using opencv under the hood, by setting: opencv=True")
                                
                        except FileNotFoundError:
                            print("Frame # ", index, " does not exist.")
                    
                    #close video. 
                    clip.close()
                    del clip
    else:
        print("Invalid MODE. Choose either 'manual' or 'automatic'. Check ``help(deeplabcut.extract_frames)`` on python and ``deeplabcut.extract_frames?`` \
              for ipython/jupyter notebook for more details.")
    
    print("\nFrames were selected.\nYou can now label the frames using the function 'label_frames' (if you extracted enough frames for all videos).")

def line_select_callback(eclick, erelease):
        'eclick and erelease are the press and release events'
        global coords
        new_x1, new_y1 = eclick.xdata, eclick.ydata
        new_x2, new_y2 = erelease.xdata, erelease.ydata
        coords = [str(int(new_x1)),str(int(new_x2)),str(int(new_y1)),str(int(new_y2))]



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut
Please see AUTHORS for contributors.

https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0

"""

import wx
import os,sys,pydoc
import deeplabcut
media_path = os.path.join(deeplabcut.__path__[0], 'gui' , 'media')
logo = os.path.join(media_path,'logo.png')

from deeplabcut.utils import auxiliaryfunctions
class Evaluate_network(wx.Panel):
    """
    """

    def __init__(self, parent,gui_size,cfg):
        """Constructor"""
        wx.Panel.__init__(self, parent=parent)

        # variable initilization
        self.config = cfg
        self.bodyparts = []
        # design the panel
        self.sizer = wx.GridBagSizer(5, 5)

        text = wx.StaticText(self, label="DeepLabCut - Step 6. Evaluate Network")
        self.sizer.Add(text, pos=(0, 0), flag=wx.TOP|wx.LEFT|wx.BOTTOM,border=15)
        # Add logo of DLC
        icon = wx.StaticBitmap(self, bitmap=wx.Bitmap(logo))
        self.sizer.Add(icon, pos=(0, 4), flag=wx.TOP|wx.RIGHT|wx.ALIGN_RIGHT,border=5)

        line1 = wx.StaticLine(self)
        self.sizer.Add(line1, pos=(1, 0), span=(1, 5),flag=wx.EXPAND|wx.BOTTOM, border=10)

        self.cfg_text = wx.StaticText(self, label="Select the config file")
        self.sizer.Add(self.cfg_text, pos=(2, 0), flag=wx.TOP|wx.LEFT, border=5)

        if sys.platform=='darwin':
            self.sel_config = wx.FilePickerCtrl(self, path="",style=wx.FLP_USE_TEXTCTRL,message="Choose the config.yaml file", wildcard="*.yaml")
        else:
            self.sel_config = wx.FilePickerCtrl(self, path="",style=wx.FLP_USE_TEXTCTRL,message="Choose the config.yaml file", wildcard="config.yaml")
        # self.sel_config = wx.FilePickerCtrl(self, path="",style=wx.FLP_USE_TEXTCTRL,message="Choose the config.yaml file", wildcard="config.yaml")
        self.sizer.Add(self.sel_config, pos=(2, 1),span=(1,3),flag=wx.TOP|wx.EXPAND, border=5)
        self.sel_config.SetPath(self.config)
        self.sel_config.Bind(wx.EVT_FILEPICKER_CHANGED, self.select_config)


        sb = wx.StaticBox(self, label="Optional Attributes")
        boxsizer = wx.StaticBoxSizer(sb, wx.VERTICAL)

        self.hbox1 = wx.BoxSizer(wx.HORIZONTAL)
#        self.hbox2 = wx.BoxSizer(wx.HORIZONTAL)

        shuffles_text = wx.StaticBox(self, label="Specify the shuffle")
        shuffles_text_boxsizer = wx.StaticBoxSizer(shuffles_text, wx.VERTICAL)
        self.shuffles = wx.SpinCtrl(self, value='1',min=1,max=100)
        shuffles_text_boxsizer.Add(self.shuffles,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        trainingset = wx.StaticBox(self, label="Specify the trainingset index")
        trainingset_boxsizer = wx.StaticBoxSizer(trainingset, wx.VERTICAL)
        self.trainingset = wx.SpinCtrl(self, value='0',min=0,max=100)
        trainingset_boxsizer.Add(self.trainingset,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        self.plot_choice = wx.RadioBox(self, label='Want to plot predictions?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.plot_choice.SetSelection(1)

        self.bodypart_choice = wx.RadioBox(self, label='Compare all bodyparts?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.bodypart_choice.Bind(wx.EVT_RADIOBOX,self.chooseOption)

        config_file = auxiliaryfunctions.read_config(self.config)
        bodyparts = config_file['bodyparts']
        self.bodyparts_to_compare = wx.CheckListBox(self, choices=bodyparts, style=0,name = "Select the bodyparts")
        self.bodyparts_to_compare.Bind(wx.EVT_CHECKLISTBOX,self.getbp)
        self.bodyparts_to_compare.Hide()
#        self.SetSizer(self.sizer)
#        self.sizer.Fit(self)

        self.hbox1.Add(shuffles_text_boxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        self.hbox1.Add(trainingset_boxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        self.hbox1.Add(self.plot_choice,5, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        self.hbox1.Add(self.bodypart_choice,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        self.hbox1.Add(self.bodyparts_to_compare,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)

        boxsizer.Add(self.hbox1,0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
#        boxsizer.Add(self.hbox2,5, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        self.sizer.Add(boxsizer, pos=(3, 0), span=(1, 5),flag=wx.EXPAND|wx.TOP|wx.LEFT|wx.RIGHT , border=10)

        self.help_button = wx.Button(self, label='Help')
        self.sizer.Add(self.help_button, pos=(4, 0), flag=wx.LEFT, border=10)
        self.help_button.Bind(wx.EVT_BUTTON, self.help_function)

        self.ok = wx.Button(self, label="Ok")
        self.sizer.Add(self.ok, pos=(4, 4))
        self.ok.Bind(wx.EVT_BUTTON, self.evaluate_network)

        self.cancel = wx.Button(self, label="Reset")
        self.sizer.Add(self.cancel, pos=(4, 1), span=(1, 1),flag=wx.BOTTOM|wx.RIGHT, border=10)
        self.cancel.Bind(wx.EVT_BUTTON, self.cancel_evaluate_network)

        self.sizer.AddGrowableCol(2)

        self.SetSizer(self.sizer)
        self.sizer.Fit(self)

    def help_function(self,event):

        filepath= 'help.txt'
        f = open(filepath, 'w')
        sys.stdout = f
        fnc_name = 'deeplabcut.evaluate_network'
        pydoc.help(fnc_name)
        f.close()
        sys.stdout = sys.__stdout__
        help_file = open("help.txt","r+")
        help_text = help_file.read()
        wx.MessageBox(help_text,'Help',wx.OK | wx.ICON_INFORMATION)
        help_file.close()
        os.remove('help.txt')

    def chooseOption(self,event):
        if self.bodypart_choice.GetStringSelection() == 'No':
            self.bodyparts_to_compare.Show()
            self.getbp(event)
            self.SetSizer(self.sizer)
            self.sizer.Fit(self)
        if self.bodypart_choice.GetStringSelection() == 'Yes':
            self.bodyparts_to_compare.Hide()
            self.SetSizer(self.sizer)
            self.sizer.Fit(self)
            self.bodyparts = 'all'

    def getbp(self,event):
        self.bodyparts = list(self.bodyparts_to_compare.GetCheckedStrings())


    def select_config(self,event):
        """
        """
        self.config = self.sel_config.GetPath()

    def evaluate_network(self,event):

        #shuffle = self.shuffle.GetValue()
        trainingsetindex = self.trainingset.GetValue()

        shuffle = [self.shuffles.GetValue()]
        if self.plot_choice.GetStringSelection() == "Yes":
            plotting = True
        else:
            plotting = False

        if len(self.bodyparts)==0:
            self.bodyparts='all'
        deeplabcut.evaluate_network(self.config,Shuffles=shuffle,trainingsetindex=trainingsetindex,plotting=plotting,show_errors=True,comparisonbodyparts=self.bodyparts,gputouse=None)

    def cancel_evaluate_network(self,event):
        """
        Reset to default
        """
        self.config = []
        self.sel_config.SetPath("")
        self.plot_choice.SetSelection(1)
        self.bodypart_choice.SetSelection(0)
        self.shuffles.SetValue(1)
        if self.bodyparts_to_compare.IsShown():
            self.bodyparts_to_compare.Hide()



'''
Adapted from DeeperCut by Eldar Insafutdinov
https://github.com/eldar/pose-tensorflow
'''
import logging, os


def setup_logging():
    FORMAT = '%(asctime)-15s %(message)s'
    logging.basicConfig(filename=os.path.join('log.txt'), filemode='w',
                        datefmt='%Y-%m-%d %H:%M:%S',
                        level=logging.INFO, format=FORMAT)
    
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    logging.getLogger('').addHandler(console)



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut
Please see AUTHORS for contributors.

https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""
from deeplabcut.pose_estimation_tensorflow.dataset.factory import *
from deeplabcut.pose_estimation_tensorflow.dataset.pose_dataset import *



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut

Please see AUTHORS for contributors.
https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""

from deeplabcut import DEBUG
import os

if os.environ.get('DLClight', default=False) == 'True':
    #print("DLC loaded in light mode; you cannot use the labeling GUI!")
    pass
else:
    from deeplabcut.generate_training_dataset.auxfun_drag_label import *
    from deeplabcut.generate_training_dataset.labeling_toolbox import *
    from deeplabcut.generate_training_dataset.multiple_individual_labeling_toolbox import *
    from deeplabcut.generate_training_dataset.frame_extraction_toolbox import *

from deeplabcut.generate_training_dataset.frame_extraction import *
from deeplabcut.generate_training_dataset.trainingsetmanipulation import *



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut

Please see AUTHORS for contributors.
https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0

Loader structure adapted from DeeperCut by Eldar Insafutdinov
https://github.com/eldar/pose-tensorflow

Uses imgaug dataflow for flexible augmentation
Largely written by Mert Yüksekgönül during the summer in the Bethge lab -- Thanks!
https://imgaug.readthedocs.io/en/latest/
"""

import os
import logging
import random as rand
import numpy as np
from numpy import array as arr
from numpy import concatenate as cat

import scipy.io as sio
from deeplabcut.utils.auxfun_videos import imread
from scipy.stats import truncnorm
from imgaug.augmentables import Keypoint, KeypointsOnImage
import imgaug.augmenters as iaa
import pickle

from deeplabcut.pose_estimation_tensorflow.dataset.pose_dataset import Batch, data_to_input, DataItem, data_to_input_batch

class PoseDataset:
    def __init__(self, cfg):
        self.cfg = cfg
        self.data = self.load_dataset()
        self.batch_size = cfg.get('batch_size',1)
        self.num_images = len(self.data)
        self.max_input_sizesquare=cfg.get('max_input_size', 1500)**2
        self.min_input_sizesquare=cfg.get('min_input_size', 64)**2
        self.locref_scale = 1.0 / cfg.locref_stdev
        self.stride = cfg.stride
        self.half_stride = cfg.stride / 2
        self.scale = cfg.global_scale
        self.scale_jitter_lo=cfg.get('scale_jitter_lo',.75)
        self.scale_jitter_up=cfg.get('scale_jitter_up',1.25)
        print("Batch Size is %d" % self.batch_size)

    def load_dataset(self):
        cfg = self.cfg
        file_name = os.path.join(self.cfg.project_path,cfg.dataset)
        if '.mat' in file_name: #legacy loader
            mlab = sio.loadmat(file_name)
            self.raw_data = mlab
            mlab = mlab['dataset']

            num_images = mlab.shape[1]
            data = []
            has_gt = True

            for i in range(num_images):
                sample = mlab[0, i]

                item = DataItem()
                item.image_id = i
                item.im_path = sample[0][0]
                item.im_size = sample[1][0]
                if len(sample) >= 3:
                    joints = sample[2][0][0]
                    joint_id = joints[:, 0]
                    # make sure joint ids are 0-indexed
                    if joint_id.size != 0:
                        assert((joint_id < cfg.num_joints).any())
                    joints[:, 0] = joint_id
                    item.joints = [joints]
                else:
                    has_gt = False
                data.append(item)

            self.has_gt = has_gt
            return data
        else:
            print("Loading pickle data with float coordinates!")
            file_name = cfg.dataset.split(".")[0] + ".pickle"
            with open(os.path.join(self.cfg.project_path,file_name), 'rb') as f:
                pickledata=pickle.load(f)

            self.raw_data = pickledata
            num_images = len(pickledata) #mlab.shape[1]
            data = []
            has_gt = True
            for i in range(num_images):
                sample = pickledata[i] #mlab[0, i]
                item = DataItem()
                item.image_id = i
                item.im_path = sample['image'] #[0][0]
                item.im_size = sample['size'] #sample[1][0]
                if len(sample) >= 3:
                    item.num_animals=len(sample['joints'])
                    item.joints=[sample['joints']]

                else:
                    has_gt = False
                data.append(item)
            self.has_gt = has_gt
            return data

    def build_augmentation_pipeline(self,
                                    height=None,
                                    width=None,
                                    apply_prob=0.5
                                   ):
        sometimes = lambda aug: iaa.Sometimes(apply_prob, aug)
        pipeline = iaa.Sequential(random_order=False)
        cfg = self.cfg
        if cfg.get('fliplr', False):
            opt = cfg.get('fliplr',False)
            if type(opt) == int:
                pipeline.add(sometimes(iaa.Fliplr(opt)))
            else:
                pipeline.add(sometimes(iaa.Fliplr(0.5)))
        if cfg.get('rotation', False):
            opt = cfg.get('rotation',False)
            if type(opt) == int:
                pipeline.add(sometimes(iaa.Affine(rotate=(-opt,opt))))
            else:
                pipeline.add(sometimes(iaa.Affine(rotate=(-10,10))))
        if cfg.get('motion_blur', False):
            opts = cfg.get('motion_blur', False)
            if type(opts) == list:
                opts = dict(opts)
                pipeline.add(sometimes(iaa.MotionBlur(**opts)))
            else:
                pipeline.add(sometimes(iaa.MotionBlur(k=7, angle=(-90, 90))))
        if cfg.get('covering', False):
            pipeline.add(sometimes(iaa.CoarseDropout(0.02, size_percent=0.3, per_channel=0.5)))
        if cfg.get('elastic_transform', False):
            pipeline.add(sometimes(iaa.ElasticTransformation(sigma=5)))
        if cfg.get('gaussian_noise', False):
            opt = cfg.get('gaussian_noise', False)
            if type(opt) == int or type(opt)==float:
                pipeline.add(sometimes(iaa.AdditiveGaussianNoise(loc=0, scale=(0.0, opt), per_channel=0.5)))
            else:
                pipeline.add(sometimes(iaa.AdditiveGaussianNoise(loc=0, scale=(0.0, 0.05*255), per_channel=0.5)))
        if cfg.get('grayscale', False):
            pipeline.add(sometimes(iaa.Grayscale(alpha=(0.5, 1.0))))

        if cfg.get('hist_eq', False):
            pipeline.add(sometimes(iaa.AllChannelsHistogramEqualization()))
        if height is not None and width is not None:
            if not cfg.get('crop_by', False):
                crop_by = 0.15
            else:
                crop_by = cfg.get('crop_by', False)
            pipeline.add(iaa.Sometimes(cfg.cropratio,iaa.CropAndPad(percent=(-crop_by, crop_by),keep_size=False)))
            pipeline.add(iaa.Resize({"height": height, "width": width}))
        return pipeline

    def get_batch(self):
        img_idx = np.random.choice(self.num_images,
                                   size=self.batch_size,replace=True)
        batch_images = []
        batch_joints = []
        joint_ids = []
        data_items = []
        # Scale is sampled only once to transform all of the images into same size.
        scale = self.get_scale()
        while True:
            idx = np.random.choice(self.num_images)
            scale = self.get_scale()
            size = self.data[idx].im_size
            target_size = np.ceil(size[1:3]*scale).astype(int)
            if self.is_valid_size(target_size[1] * target_size[0]):
                break

        stride = self.cfg.stride
        for i in range(self.batch_size):
            data_item = self.data[img_idx[i]]

            data_items.append(data_item)
            im_file = data_item.im_path

            logging.debug('image %s', im_file)
            image = imread(os.path.join(self.cfg.project_path,im_file), mode='RGB')

            if self.has_gt:
                joints = np.copy(data_item.joints)
                joint_id = [person_joints[:, 0].astype(int) for person_joints in joints]
                joint_points = [person_joints[:, 1:3] for person_joints in joints]
                joint_ids.append(joint_id)
                batch_joints.append(arr(joint_points)[0])
            batch_images.append(image)
        sm_size = np.ceil(target_size / (stride * 2)).astype(int) * 2
        assert len(batch_images) == self.batch_size
        return batch_images, joint_ids, batch_joints, data_items, sm_size, target_size

    def get_scmap_update(self, joint_ids, joints, data_items, sm_size, target_size):
        part_score_targets, part_score_weights, locref_targets, locref_masks = [],[],[],[]
        for i in range(len(data_items)):
            # Approximating the scale
            scale = min(target_size[0]/data_items[i].im_size[1], target_size[1]/data_items[i].im_size[2])
            if self.cfg.get("scmap_type", None) == "gaussian":
                part_score_target, part_score_weight, locref_target, locref_mask = self.gaussian_scmap(
                    joint_ids[i], [joints[i]], data_items[i], sm_size, scale)
            else:
                part_score_target, part_score_weight, locref_target, locref_mask = self.compute_target_part_scoremap_numpy(
                    joint_ids[i], [joints[i]], data_items[i], sm_size, scale)
            part_score_targets.append(part_score_target)
            part_score_weights.append(part_score_weight)
            locref_targets.append(locref_target)
            locref_masks.append(locref_mask)

        return {
                    Batch.part_score_targets: part_score_targets,
                    Batch.part_score_weights: part_score_weights,
                    Batch.locref_targets: locref_targets,
                    Batch.locref_mask: locref_masks
                }

    def next_batch(self):
        while True:
            batch_images, joint_ids, batch_joints, data_items, sm_size, target_size = self.get_batch()
            pipeline = self.build_augmentation_pipeline(
                                    height=target_size[0],
                                    width=target_size[1],
                                    apply_prob=0.5)
            batch_images, batch_joints = pipeline(images=batch_images,
                                                  keypoints=batch_joints)
            #If you would like to check the augmented images, script for saving
            #the images with joints on:
            #import imageio
            #for i in range(self.batch_size):
            #    joints = batch_joints[i]
            #    kps = KeypointsOnImage([Keypoint(x=joint[0], y=joint[1]) for joint in joints], shape=batch_images[i].shape)
            #    im = kps.draw_on_image(batch_images[i])
            #    imageio.imwrite('some_location/augmented/'+str(i)+'.png', im)

            image_shape = arr(batch_images).shape[1:3]
            batch = {Batch.inputs: arr(batch_images).astype(np.float64)}
            if self.has_gt:
                scmap_update = self.get_scmap_update(joint_ids, batch_joints, data_items, sm_size, image_shape)
                batch.update(scmap_update)

            batch = {key: data_to_input_batch(data) for (key, data) in batch.items()}
            batch[Batch.data_item] = data_items
            return batch

    def set_test_mode(self, test_mode):
        self.has_gt = not test_mode

    def num_training_samples(self):
        num = self.num_images
        if self.cfg.mirror:
            num *= 2
        return num

    def get_scale(self):
        cfg = self.cfg
        scale = cfg.global_scale
        if hasattr(cfg, 'scale_jitter_lo') and hasattr(cfg, 'scale_jitter_up'):
            scale_jitter = rand.uniform(0.75*cfg.scale_jitter_lo, 1.25*cfg.scale_jitter_up)
            scale *= scale_jitter
        return scale

    def is_valid_size(self, target_size_product):
        if target_size_product > self.max_input_sizesquare:
            return False

        if target_size_product  < self.min_input_sizesquare:
            return False

        return True

    def gaussian_scmap(self, joint_id, coords, data_item, size, scale):
        #dist_thresh = float(self.cfg.pos_dist_thresh * scale)
        num_joints = self.cfg.num_joints
        scmap = np.zeros(cat([size, arr([num_joints])]))
        locref_size = cat([size, arr([num_joints * 2])])
        locref_mask = np.zeros(locref_size)
        locref_map = np.zeros(locref_size)

        width = size[1]
        height = size[0]
        dist_thresh = float((width+height)/6)
        dist_thresh_sq = dist_thresh ** 2

        std = dist_thresh/4
        # Grid of coordinates
        grid = np.mgrid[:height, :width].transpose((1,2,0))
        grid = grid*self.stride + self.half_stride
        for person_id in range(len(coords)):
            for k, j_id in enumerate(joint_id[person_id]):
                joint_pt = coords[person_id][k, :]
                j_x = np.asscalar(joint_pt[0])
                j_x_sm = round((j_x - self.half_stride) / self.stride)
                j_y = np.asscalar(joint_pt[1])
                j_y_sm = round((j_y - self.half_stride) / self.stride)
                map_j = grid.copy()
                # Distance between the joint point and each coordinate
                dist = np.linalg.norm(grid - (j_y, j_x), axis=2)**2
                scmap_j = np.exp(-dist/(2*(std**2)))
                scmap[..., j_id] = scmap_j
                locref_mask[dist<=dist_thresh_sq,j_id * 2 + 0]=1
                locref_mask[dist<=dist_thresh_sq,j_id * 2 + 1]=1
                dx = j_x - grid.copy()[:, :, 1 ]
                dy = j_y - grid.copy()[:, :, 0 ]
                locref_map[..., j_id * 2 + 0] = dx * self.locref_scale
                locref_map[..., j_id * 2 + 1] = dy * self.locref_scale
        weights = self.compute_scmap_weights(scmap.shape, joint_id, data_item)
        return scmap, weights, locref_map, locref_mask

    def compute_scmap_weights(self, scmap_shape, joint_id, data_item):
        if self.cfg.weigh_only_present_joints:
            weights = np.zeros(scmap_shape)
            for person_joint_id in joint_id:
                for j_id in person_joint_id:
                    weights[:, :, j_id] = 1.0
        else:
            weights = np.ones(scmap_shape)
        return weights

    def compute_target_part_scoremap_numpy(self, joint_id, coords, data_item, size, scale):
        dist_thresh = float(self.cfg.pos_dist_thresh * scale)
        dist_thresh_sq = dist_thresh ** 2
        num_joints = self.cfg.num_joints

        scmap = np.zeros(cat([size, arr([num_joints])]))
        locref_size = cat([size, arr([num_joints * 2])])
        locref_mask = np.zeros(locref_size)
        locref_map = np.zeros(locref_size)

        width = size[1]
        height = size[0]
        grid = np.mgrid[:height, :width].transpose((1,2,0))

        for person_id in range(len(coords)):
            for k, j_id in enumerate(joint_id[person_id]):
                joint_pt = coords[person_id][k, :]
                j_x = np.asscalar(joint_pt[0])
                j_x_sm = round((j_x - self.half_stride) / self.stride)
                j_y = np.asscalar(joint_pt[1])
                j_y_sm = round((j_y - self.half_stride) / self.stride)
                min_x = round(max(j_x_sm - dist_thresh - 1, 0))
                max_x = round(min(j_x_sm + dist_thresh + 1, width - 1))
                min_y = round(max(j_y_sm - dist_thresh - 1, 0))
                max_y = round(min(j_y_sm + dist_thresh + 1, height - 1))
                x = grid.copy()[:, :, 1]
                y = grid.copy()[:, :, 0]
                dx = j_x - x*self.stride - self.half_stride
                dy = j_y - y*self.stride - self.half_stride
                dist = dx**2 + dy**2
                mask1 = (dist <= dist_thresh_sq)
                mask2 = ((x >= min_x) & (x <= max_x))
                mask3 = ((y >= min_y) & (y <= max_y))
                mask = mask1 & mask2 & mask3
                scmap[mask, j_id] = 1
                locref_mask[mask, j_id*2+0] = 1
                locref_mask[mask, j_id*2+1] = 1
                locref_map[mask, j_id * 2 + 0] = (dx * self.locref_scale)[mask]
                locref_map[mask, j_id * 2 + 1] = (dy * self.locref_scale)[mask]

        weights = self.compute_scmap_weights(scmap.shape, joint_id, data_item)
        return scmap, weights, locref_map, locref_mask



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut
Please see AUTHORS for contributors.

https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""

from __future__ import print_function
import wx
import cv2
import matplotlib
import argparse
from deeplabcut.utils import auxiliaryfunctions
from matplotlib.figure import Figure
from matplotlib.backends.backend_wxagg import FigureCanvasWxAgg as FigureCanvas
from matplotlib.widgets import RectangleSelector

# ###########################################################################
# Class for GUI MainFrame
# ###########################################################################
class ImagePanel(wx.Panel):

    def __init__(self, parent,config,gui_size,**kwargs):
        h=gui_size[0]/2
        w=gui_size[1]/3
        wx.Panel.__init__(self, parent, -1,style=wx.SUNKEN_BORDER,size=(h,w))

        self.figure = matplotlib.figure.Figure()
        self.axes = self.figure.add_subplot(1, 1, 1)
        self.canvas = FigureCanvas(self, -1, self.figure)
        self.sizer = wx.BoxSizer(wx.VERTICAL)
        self.sizer.Add(self.canvas, 1, wx.LEFT | wx.TOP | wx.GROW)
        self.SetSizer(self.sizer)
        self.Fit()

    def getfigure(self):
        """
        Returns the figure, axes and canvas
        """
        return(self.figure,self.axes,self.canvas)

class WidgetPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1,style=wx.SUNKEN_BORDER)

class MainFrame(wx.Frame):
    """Contains the main GUI and button boxes"""

    def __init__(self, parent,config,image):
# Settting the GUI size and panels design
        displays = (wx.Display(i) for i in range(wx.Display.GetCount())) # Gets the number of displays
        screenSizes = [display.GetGeometry().GetSize() for display in displays] # Gets the size of each display
        index = 0 # For display 1.
        screenWidth = screenSizes[index][0]
        screenHeight = screenSizes[index][1]
        self.gui_size = (screenWidth*0.7,screenHeight*0.85)

        wx.Frame.__init__ ( self, parent, id = wx.ID_ANY, title = 'DeepLabCut2.0 - Select Crop Parameters',
                            size = wx.Size(self.gui_size), pos = wx.DefaultPosition, style = wx.RESIZE_BORDER|wx.DEFAULT_FRAME_STYLE|wx.TAB_TRAVERSAL )
        self.statusbar = self.CreateStatusBar()
        self.statusbar.SetStatusText("")

        self.SetSizeHints(wx.Size(self.gui_size)) #  This sets the minimum size of the GUI. It can scale now!
        
###################################################################################################################################################
# Spliting the frame into top and bottom panels. Bottom panels contains the widgets. The top panel is for showing images and plotting!
        topSplitter = wx.SplitterWindow(self)

        self.image_panel = ImagePanel(topSplitter, config,self.gui_size)
        self.widget_panel = WidgetPanel(topSplitter)
        
        topSplitter.SplitHorizontally(self.image_panel, self.widget_panel,sashPosition=self.gui_size[1]*0.83)#0.9
        topSplitter.SetSashGravity(1)
        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(topSplitter, 1, wx.EXPAND)
        self.SetSizer(sizer)

###################################################################################################################################################
# Add Buttons to the WidgetPanel and bind them to their respective functions.

        widgetsizer = wx.WrapSizer(orient=wx.HORIZONTAL)

        self.help = wx.Button(self.widget_panel, id=wx.ID_ANY, label="Help")
        widgetsizer.Add(self.help , 1, wx.ALL, 15)
        self.help.Bind(wx.EVT_BUTTON, self.helpButton)

        self.quit = wx.Button(self.widget_panel, id=wx.ID_ANY, label="Save parameters and Quit")
        widgetsizer.Add(self.quit , 1, wx.ALL, 15)
        self.quit.Bind(wx.EVT_BUTTON, self.quitButton)

        self.widget_panel.SetSizer(widgetsizer)
        self.widget_panel.SetSizerAndFit(widgetsizer)
        self.widget_panel.Layout()
        
# Variables initialization
        self.image = image
        self.coords = []
        self.figure = Figure()
        self.axes = self.figure.add_subplot(111)
        self.cfg = auxiliaryfunctions.read_config(config)
        MainFrame.show_image(self)

    def quitButton(self, event):
        """
        Quits the GUI
        """
        self.statusbar.SetStatusText("")
        dlg = wx.MessageDialog(None,"Are you sure?", "Quit!",wx.YES_NO | wx.ICON_WARNING)
        result = dlg.ShowModal()
        if result == wx.ID_YES:
            self.Destroy()

    def show_image(self):
        self.figure,self.axes,self.canvas = self.image_panel.getfigure()
        self.ax = self.axes.imshow(self.image)
        self.figure.canvas.draw()
        self.cid = RectangleSelector(self.axes, self.line_select_callback,drawtype='box', useblit=False,button=[1], minspanx=5, minspany=5,spancoords='pixels',interactive=True)
        self.canvas.mpl_connect('key_press_event', self.cid)
        
        
    def line_select_callback(self,eclick, erelease):
        'eclick and erelease are the press and release events'
        global coords
        new_x1, new_y1 = eclick.xdata, eclick.ydata
        new_x2, new_y2 = erelease.xdata, erelease.ydata
        coords = [str(int(new_x1)),str(int(new_x2)),str(int(new_y1)),str(int(new_y2))]
        self.coords = coords
        return(self.coords)
        
    def helpButton(self,event):
        """
        Opens Instructions
        """
        wx.MessageBox('1. Use left click to select the region of interest. A red box will be drawn around the selected region. \n\n2. Use the corner points to expand the box and center to move the box around the image. \n\n3. Click ''Save parameters and Quit'' to save the croppeing parameters and close the GUI. \n\n Click OK to continue', 'Instructions to use!', wx.OK | wx.ICON_INFORMATION)

def show(config,image):
    app = wx.App()
    MainFrame(None,config,image).Show()
    app.MainLoop()
    return(coords)


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('config','image')
    cli_args = parser.parse_args()



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut
Please see AUTHORS for contributors.

https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""
import os
import click
from pathlib import Path

import deeplabcut

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])


@click.group(invoke_without_command=True)
# @click.version_option()
@click.option('-v', '--verbose', is_flag=True, help='Verbose printing')
@click.pass_context
def main(ctx, verbose):
    if ctx.invoked_subcommand is None:
        click.echo('deeplabcut v0.0.')
        click.echo(main.get_help(ctx))

###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('project')
@click.argument('experimenter')
@click.argument('videos', nargs=-1, type=click.Path(exists=True, dir_okay=False))
@click.option('-d', '--wd', 'working_directory',
              type=click.Path(exists=True, file_okay=False, resolve_path=True), default=Path.cwd(),
              help='Directory to create project in. Default is cwd().')
@click.option('--copy_videos/--dont_copy_videos',
             is_flag=True,
             default=True,
             help='Specify if you need to create the symlinks of the video and store in the videos directory. Default is True.')
#              type=click.Path(exists=True, file_okay=False, resolve_path=True), default=Path.cwd(),
#              help='Directory to create project in. Default is cwd().')
@click.pass_context

def create_new_project(_, *args, **kwargs):
    """Create a new project directory, sub-directories and a basic configuration file. The configuration file is loaded with default values. Change its parameters to your projects need.\n

    Options \n
    ---------- \n
    project : string \n
    \tString containing the name of the project.\n
    experimenter : string \n
    \tString containing the name of the experimenter. \n
    videos : list \n
    \tA list of string containing the full paths of the videos to include in the project.\n
    working_directory : string, optional \n
    \tThe directory where the project will be created. The default is the ``current working directory``; if provided, it must be a string\n
    copy_videos : bool, optional \n
    If this is set to True, the symlink of the videos are copied to the project/videos directory. The default is ``True``; if provided it must be either ``True`` or ``False`` \n

    Example \n
    -------- \n
    To create the project in the current working directory \n
    python3 dlc.py create_new_project reaching-task Tanmay /data/videos/mouse1.avi /data/videos/mouse2.avi /data/videos/mouse3.avi /analysis/project/

    To create the project in the current working directory but do not want to create the symlinks \n
    python3 dlc.py create_new_project reaching-task Tanmay /data/videos/mouse1.avi /data/videos/mouse2.avi /data/videos/mouse3.avi /analysis/project/ -c False

    To create the project in another directory \n
    python3 dlc.py create_new_project reaching-task Tanmay /data/vies/mouse1.avi /data/videos/mouse2.avi /data/videos/mouse3.avi analysis/project -d home/project

    """
    from deeplabcut.create_project import new
    new.create_new_project(*args, **kwargs)

###########################################################################################################################

@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.argument('videos', nargs=-1, type=click.Path(exists=True, dir_okay=False))
@click.option('--copy_videos/--dont_copy_videos',
             is_flag=True,
             default=True,
             help='Specify if you need to create the symlinks of the video and store in the videos directory. Default is True.')
@click.pass_context
def add_new_videos(_, *args, **kwargs):
    """
    Add new videos to the config file at any stage of the project.\n

    Options\n
    ----------\n
    config : string\n
        String containing the full path of the config file in the project.

    videos : list \n
        A list of string containing the full paths of the videos to include in the project.

    copy_videos : bool, optional\n
        If this is set to True, the symlink of the videos are copied to the project/videos directory. The default is
        ``True``; if provided it must be either ``True`` or ``False``

    Examples\n
    --------\n
    >>> python3 dlc.py add_new_videos /home/project/reaching-task-Tanmay-2018-08-23/config.yaml /data/videos/mouse5.avi

    """
    from deeplabcut.create_project import add
    add.add_new_videos(*args, **kwargs)

###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.argument('mode')
@click.option('-a', '--algo', 'algo',
              default='uniform',
              help='For automatic extraction, specify the algorithm- "kmeans" or "uniform". Default is uniform.')
@click.option('--crop',
              is_flag=True,
              default=False,
              help='Specify if you need to crop the image. Default is True.')
@click.pass_context
def extract_frames(_,*args,**kwargs):
    """
    Extracts frames from the videos in the config.yaml file. Only the videos in the config.yaml will be used to select the frames.\n
    Use the function ``add_new_video`` at any stage of the project to add new videos to the config file and extract their frames. \n

    CONFIG : string \n
        Full path of the config.yaml file as a string.  \n \n \n
    MODE : string \n \n
        String containing the mode of extraction. It must be either ``automatic`` or ``manual``.  \n

    Examples \n
    -------- \n
    for selecting frames automatically with 'kmeans' and do not want to crop the frames \n
    >>> python3 dlc.py extract_frames /analysis/project/reaching-task/config.yaml automatic --algo kmeans \n
    -------- \n
    for selecting frames automatically with 'uniform' and want to crop the frames based on the ``crop`` parameters in config.yaml \n
    >>> python3 dlc.py extract_frames /analysis/project/reaching-task/config.yaml automatic --crop
    -------- \n
    for selecting frames manually, \n
    >>> deeplabcut.extract_frames /analysis/project/reaching-task/config.yaml manual \n
    While selecting the frames manually, you do not need to specify the cropping parameters. Rather, you will get a prompt in the graphic user interface to choose if you need to crop or not. \n
    -------- \n

    """
    from deeplabcut.generate_training_dataset import frameExtraction
    frameExtraction.extract_frames(*args,**kwargs)

###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.pass_context
def label_frames(_,config):
    """Manually label/annotate the extracted frames. Update the list of body parts you want to localize in the config.yaml file first.\n
    Example\n
    --------\n
    python3 dlc.py label_frames /analysis/project/reaching-task/config.yaml
    """
    from deeplabcut.generate_training_dataset import labelFrames
    labelFrames.label_frames(config)

###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.pass_context
def check_labels(_,config):
    """Check if labels were stored correctly by plotting annotations and inspect them visually. If some are wrong, then use the refine_labels to correct the labels.\n

    """
    from deeplabcut.generate_training_dataset import labelFrames
    labelFrames.check_labels(config)
###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.option('-num', '--num_shuffles', 'num_shuffles',
              default=1,
              help='Number of shuffles of training dataset to create. Default is set to 1.')
@click.pass_context

def create_training_dataset(_,*args,**kwargs):
    """Combine frame and label information into a an array. Create training and test sets. Update parameters TrainFraction, iteration in config.yaml
	Also update parameters for pose_config.yaml as wanted.\n
    CONFIG: Full path of the config.yaml file in the train directory of a project.\n
    Example \n
    --------\n
    To create a training dataset with only 1 shuffle
    python3 dlc.py create_training_dataset /analysis/project/reaching-task/config.yaml

    To create a training dataset with only 2 shuffles
    python3 dlc.py create_training_dataset /analysis/project/reaching-task/config.yaml num_shuffles 2
    """
    from deeplabcut.generate_training_dataset import labelFrames
    labelFrames.create_training_dataset(*args,**kwargs)
###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.option('-num', '--num_shuffles', 'shuffle',
              default=1,
              help='Shuffle index of the training dataset. Default is set to 1.')
@click.pass_context
def train_network(_,*args,**kwargs):
    """Train a trained Feature detector with a specific training data set.\n
	Provide path to the pose_config file.
        CONFIG: Full path of the config.yaml file in the train directory of a project.\n

    e.g. run the script like this:
    python3 dlc.py step7_train  /home/project/reaching/config.yaml

    """
    from deeplabcut.pose_estimation_tensorflow import training
    training.train_network(*args,**kwargs)

###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.option('-num', '--num_shuffles', 'shuffle',
              default=[1],
              help='Shuffle index of the training dataset. Default is set to 1.')
@click.option('-p','--plot','plotting',
              is_flag=True,
              help='Make plots. Default is False.')
@click.pass_context
def evaluate_network(_,config,**kwargs):
    """Evaluates a trained Feature detector model.\n
        CONFIG: Full path of the "pose_config.yaml" file in the train directory of a project.\n

    Example\n
    ----------
    Evalaute the network
    python3 dlc.py evaluate_network  /home/project/reaching/config.yaml

    """
    from deeplabcut.pose_estimation_tensorflow import evaluate
    evaluate.evaluate_network(config,**kwargs)

###########################################################################################################################

@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.argument('video', nargs=-1)
@click.option('-num', '--num_shuffles', 'shuffle',
              default=1,
              help='Shuffle index of the training dataset. Default is set to 1.')
@click.option('-vtype', '--video_type', 'videotype',
              default='.avi',
              help='The extension of video in case the input is a directory')
@click.option('-c','--save','save_as_csv',
              is_flag=True,
              help='Saves as a .csv file. Default is False.')
@click.pass_context
def analyze_videos(_, *args,**kwargs):

    """Makes prediction.\n
        CONFIG: Full path of the "config.yaml" file in the train directory of a project.\n
        VIDEOS: Full path to video.\n

    Example\n
    ----------

    python3 dlc.py analyze_videos /home/project/reaching/config.yaml /home/project/reaching/newVideo/1.avi

    """
    from deeplabcut.pose_estimation_tensorflow import predict_videos
    predict_videos.analyze_videos(*args,**kwargs)

    # for video in videos:
    #     predict.predict_video(config, video,**kwargs)
###########################################################################################################################

@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.argument('video')
@click.option('-num', '--num_shuffles', 'shuffle',
              default=1,
              help='The shuffle index of training dataset. The extracted frames will be stored in the labeled-dataset for the corresponding shuffle of training dataset. Default is set to 1')
@click.option('-outlier', '--outlier_algo', 'outlieralgorithm',
              default='fitting',
              help='String specifying the algorithm used to detect the outliers. Currently, deeplabcut supports only sarimax (this will be updated). \
              This method fits a Seasonal AutoRegressive Integrated Moving Average with eXogenous regressors model to data and computes confidence interval. \
              Based on the fraction of data points outside the confidence interval and the average distance (compared to delta) \
              the user can identify potential outlier frames. The default is set to ``fitting``. Other choices: `fitting`, `jump`, `uncertain`')
@click.option('-compare','--comparisonbodyparts','comparisonbodyparts',
              default= 'all',
              help='This select the body parts for which the comparisons with the outliers are carried out. Either ``all``, \
              then all body parts from config.yaml are used orr a list of strings that are a subset of the full list.\
               E.g. [`hand`,`Joystick`] for the demo Reaching-Mackenzie-2018-08-30/config.yaml to select only these two body parts.')
@click.option('-e', '--epsilon', 'epsilon',
              default=20,
              help='Meaning depends on outlieralgoritm. The default is set to 20 pixels.For outlieralgorithm `fitting`: \
              Float bound according to which frames are picked when the (average) body part estimate deviates from model fit. \
              For outlieralgorithm `jump`: Float bound specifying the distance by which body points jump from one frame to next (Euclidean distance)')
@click.option('-p', '--p_bound', 'p_bound',
              default=0.01,
              help='For outlieralgorithm `uncertain` this parameter defines the likelihood below, below which a body part will be flagged as a putative outlier.')
@click.option('-ard', '--ar_degree', 'ARdegree',
              default=7,
              help='For outlieralgorithm `fitting`: Autoregressive degree of Sarimax model degree. \
              See https://www.statsmodels.org/dev/generated/statsmodels.tsa.statespace.sarimax.SARIMAX.html')
@click.option('-mad', '--ma_degree', 'MAdegree',
              default=1,
              help='Int value. For outlieralgorithm `fitting`: MovingAvarage degree of Sarimax model degree.\
               See https://www.statsmodels.org/dev/generated/statsmodels.tsa.statespace.sarimax.SARIMAX.html')
@click.option('-a', '--alpha', 'alpha',
              default=0.01,
              help='Significance level for detecting outliers based on confidence interval of fitted SARIMAX model.')
@click.option('-extract', '--extraction_algo', 'extractionalgorithm',
              default='uniform',
              help='String specifying the algorithm to use for selecting the frames from the identified outliers. \
              Currently, deeplabcut supports either ``kmeans`` or ``uniform`` based selection (same logic as for extract_frames).\
              The default is set to``uniform``, if provided it must be either ``uniform`` or ``kmeans``.')
@click.pass_context

def extract_outlier_frames(_,*args,**kwargs):
    """
    Extracts the outlier frames in case, the predictions are not correct for a certain video from the cropped video running from
    start to stop as defined in config.yaml.

    Another crucial parameter in config.yaml is how many frames to extract 'numframes2extract'.

    CONFIG : string \n
    Full path of the config.yaml file as a string.  \n
    VIDEO : Full path of the video to extract the frame from. Make sure that this video is already analyzed.


    Example \n
    --------\n
    for extracting the frames with default settings\n
    >>> python3 dlc.py extract_outlier_frames /analysis/project/reaching-task/config.yaml /analysis/project/video/reachinvideo1.avi \n
    --------\n
    for extracting the frames with kmeans\n
    >>> python3 dlc.py extract_outlier_frames /analysis/project/reaching-task/config.yaml /analysis/project/video/reachinvideo1.avi --extractionalgorithm 'kmeans' \n
    --------\n
    for extracting the frames with kmeans and epsilon = 5 pixels.\n
    >>> python3 dlc.py extract_outlier_frames /analysis/project/reaching-task/config.yaml /analysis/project/video/reachinvideo1.avi --epsilon 5 --extractionalgorithm kmeans \n
    --------\n

    """
    from deeplabcut.refine_training_dataset import outlier_frames
    outlier_frames.extract_outlier_frames(*args,**kwargs)

###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.pass_context
def refine_labels(_,config):
    """Refines the labels of the outlier frames extracted from the analyzed videos.\n Helps in augmenting the training dataset.
    Use the function ``analyze_video`` to analyze a video and extracts the outlier frames using the function
    ``extract_outlier_frames`` before refining the labels.\n

    Examples \n
    --------\n
    >>> python3 dlc.py refine_labels /analysis/project/reaching-task/config.yaml \n
    --------\n
    """
    from deeplabcut.refine_training_dataset import outlier_frames
    outlier_frames.refine_labels(config)

###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.argument('video',nargs=-1)
@click.option('-num', '--num_shuffles', 'shuffle',
              default= 1,
              help = 'Number of shuffles of training dataset. Default is set to 1.')
@click.option('-v', '--video_type', 'videotype',
              default= '.avi',
              help = 'Checks for the extension of the video in case the input is a directory.\
              Only videos with this extension are analyzed. The default is ``.avi``')
@click.option('-s', '--save_frames', 'save_frames',
              is_flag=True,
              default=False,
              help = 'If true creates each frame individual and then combines into a video. \
              This variant is relatively slow as it stores all individual frames. However, it \
              uses matplotlib to create the frames and is therefore much more flexible \
              (one can set transparency of markers, crop, and easily customize.')
@click.option('-d', '--delete', 'delete',
              is_flag=True,
              default=False,
              help = 'If true then the individual frames created during the video generation will be deleted.\
              Only the video will be left.')
@click.pass_context
def create_labeled_video(_,*args,**kwargs):
    """
    Labels the bodyparts in a video. Make sure the video is already analyzed by the function 'analyze_video'
    """
    from deeplabcut.utils import make_labeled_video
    make_labeled_video.create_labeled_video(*args,**kwargs)

###########################################################################################################################
@main.command(context_settings=CONTEXT_SETTINGS)
@click.argument('config')
@click.argument('video',nargs=-1)
@click.option('-num', '--num_shuffles', 'shuffle',
              default= 1,
              help = 'Number of shuffles of training dataset. Default is set to 1.')
@click.option('-v', '--video_type', 'videotype',
              default= '.avi',
              help = 'Checks for the extension of the video in case the input is a directory.\
              Only videos with this extension are analyzed. The default is ``.avi``')
@click.option('-s', '--show', 'showfigures',
              is_flag=True,
              default=False,
              help = 'If true then plots are also displayed simultaneously.')
@click.pass_context
def plot_trajectories(_,*args,**kwargs):
    """
    Plots the trajectories of various bodyparts across the video.\n

    Example\n
    --------\n
    for labeling the frames\n
    >>> python3 dlc.py plot_trajectories /analysis/project/reaching-task/config.yaml /analysis/project/videos/reachingvideo1.avi  \n
    --------\n
    """
    from deeplabcut.utils import plotting
    plotting.plot_trajectories(*args, **kwargs)

###########################################################################################################################



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut

Please see AUTHORS for contributors.
https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""

import os
#import yaml
from pathlib import Path
#import cv2
from deeplabcut import DEBUG
import matplotlib.pyplot as plt
#import shutil

def create_new_project_3d(project, experimenter, num_cameras=2, working_directory=None):
    """Creates a new project directory, sub-directories and a basic configuration file for 3d project. 
    The configuration file is loaded with the default values. Adjust the parameters to your project's needs.

    Parameters
    ----------
    project : string
        String containing the name of the project.

    experimenter : string
        String containing the name of the experimenter.

    num_cameras : int
        An integer value specifying the number of cameras.
        
    working_directory : string, optional
        The directory where the project will be created. The default is the ``current working directory``; if provided, it must be a string.


    Example
    --------
    Linux/MacOs
    >>> deeplabcut.create_new_project_3d('reaching-task','Linus',2)

    Windows:
    >>> deeplabcut.create_new_project('reaching-task','Bill',2)
    Users must format paths with either:  r'C:\ OR 'C:\\ <- i.e. a double backslash \ \ )

    """
    from datetime import datetime as dt
    from deeplabcut.utils import auxiliaryfunctions
    
    date = dt.today()
    month = date.strftime("%B")
    day = date.day
    d = str(month[0:3]+str(day))
    date = dt.today().strftime('%Y-%m-%d')
    
    if working_directory == None:
        working_directory = '.'
    
    wd = Path(working_directory).resolve()
    project_name = '{pn}-{exp}-{date}-{triangulate}'.format(pn=project, exp=experimenter, date=date,triangulate='3d')
    project_path = wd / project_name
    # Create project and sub-directories
    if not DEBUG and project_path.exists():
        print('Project "{}" already exists!'.format(project_path))
        return
    
    camera_matrix_path = project_path / 'camera_matrix'
    calibration_images_path = project_path / 'calibration_images'
    undistortion_path = project_path / 'undistortion'
    path_corners = project_path/'corners'
    
    for p in [camera_matrix_path, calibration_images_path, undistortion_path,path_corners]:
        p.mkdir(parents=True, exist_ok=DEBUG)
        print('Created "{}"'.format(p))
    
    # Create config file
    cfg_file_3d,ruamelFile_3d = auxiliaryfunctions.create_config_template_3d()
    cfg_file_3d['Task']=project
    cfg_file_3d['scorer']=experimenter
    cfg_file_3d['date']=d
    cfg_file_3d['project_path']= str(project_path)
#    cfg_file_3d['config_files']= [str('Enter the path of the config file ')+str(i)+ ' to include' for i in range(1,3)]
#    cfg_file_3d['config_files']= ['Enter the path of the config file 1']
    cfg_file_3d['colormap'] = 'jet'
    cfg_file_3d['dotsize'] =  15
    cfg_file_3d['alphaValue'] = 0.8
    cfg_file_3d['markerType'] = '*'
    cfg_file_3d['markerColor'] = 'r'
    cfg_file_3d['pcutoff'] = 0.4
    cfg_file_3d['num_cameras'] = num_cameras
    cfg_file_3d['camera_names'] = [str('camera-'+str(i))for i in range(1,num_cameras+1)]
    cfg_file_3d['scorername_3d'] = 'DLC_3D'
    
    cfg_file_3d['skeleton'] = [['bodypart1','bodypart2'], ['bodypart2','bodypart3'], ['bodypart3','bodypart4'], ['bodypart4','bodypart5']]
    cfg_file_3d['skeleton_color'] = 'black'
    
    for i in range(num_cameras):
        path = str('/home/mackenzie/DEEPLABCUT/DeepLabCut/2DprojectCam'+str(i+1)+'-Mackenzie-2019-06-05/config.yaml')
        cfg_file_3d.insert(len(cfg_file_3d), str('config_file_camera-'+str(i+1)),path)
    
    for i in range(num_cameras):
        cfg_file_3d.insert(len(cfg_file_3d), str('shuffle_camera-'+str(i+1)),1)
        cfg_file_3d.insert(len(cfg_file_3d), str('trainingsetindex_camera-'+str(i+1)),0)
        
    projconfigfile=os.path.join(str(project_path),'config.yaml')
    auxiliaryfunctions.write_config_3d(projconfigfile,cfg_file_3d)
    
    print('Generated "{}"'.format(project_path / 'config.yaml'))
    print("\nA new project with name %s is created at %s and a configurable file (config.yaml) is stored there. If you have not calibrated the cameras, then use the function 'calibrate_camera' to start calibrating the camera otherwise use the function ``triangulate`` to triangulate the dataframe"%(project_name,wd))
    return projconfigfile



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut
Please see AUTHORS for contributors.

https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""

import numpy as np
import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D
import cv2
import os
import pickle
import pandas as pd
from pathlib import Path
import glob

def Foldernames3Dproject(cfg_3d):
    ''' Definitions of subfolders in 3D projects '''
    
    img_path = os.path.join(cfg_3d['project_path'],'calibration_images')
    path_corners = os.path.join(cfg_3d['project_path'],'corners')
    path_camera_matrix = os.path.join(cfg_3d['project_path'],'camera_matrix')
    path_undistort = os.path.join(cfg_3d['project_path'],'undistortion')
    
    return img_path,path_corners,path_camera_matrix,path_undistort

def create_empty_df(dataframe,scorer,flag):
    # Creates an empty dataFrame of same shape as df_side_view
    # flag = 2d or 3d

    df = dataframe
    bodyparts =  df.columns.get_level_values(1)
    _, idx = np.unique(bodyparts, return_index=True)
    bodyparts =  list(bodyparts[np.sort(idx)])
    a = np.empty((df.shape[0],3))
    a[:] = np.nan
    dataFrame = None
    for bodypart in bodyparts:
        if flag == '2d':
            pdindex = pd.MultiIndex.from_product([[scorer], [bodypart], ['x', 'y','likelihood']],
                                             names=['scorer', 'bodyparts', 'coords'])
        elif flag == '3d':
            pdindex = pd.MultiIndex.from_product([[scorer], [bodypart], ['x', 'y','z']],
                                             names=['scorer', 'bodyparts', 'coords']) 
        frame = pd.DataFrame(a, columns = pdindex,index = range(0,df.shape[0]))
        dataFrame = pd.concat([frame,dataFrame],axis=1)
    return(dataFrame,scorer,bodyparts)

def compute_triangulation_calibration_images(stereo_matrix, projectedPoints1, projectedPoints2,path_undistort,cfg_3d,plot= True):
    """
    Performs triangulation of the calibration images.
    """
    triangulate = []
    P1 = stereo_matrix['P1']
    P2 = stereo_matrix['P2']
    cmap = cfg_3d['colormap']
    colormap = plt.get_cmap(cmap)
    markerSize = cfg_3d['dotsize']
    markerType = cfg_3d['markerType']
    
    for i in range(projectedPoints1.shape[0]):
        X_l = triangulatePoints(P1, P2, projectedPoints1[i], projectedPoints2[i])
        triangulate.append(X_l)
    triangulate = np.asanyarray(triangulate)

# Plotting
    if plot == True:
        col = colormap(np.linspace(0, 1, triangulate.shape[0]))
        fig = plt.figure()
        ax = fig.add_subplot(111, projection='3d')

        for i in range(triangulate.shape[0]):
            xs = triangulate[i,0,:]
            ys = triangulate[i,1,:]
            zs = triangulate[i,2,:]
            ax.scatter(xs, ys, zs,c=col[i],marker=markerType,s=markerSize)
            ax.set_xlabel('X')
            ax.set_ylabel('Y')
            ax.set_zlabel('Z')
        plt.savefig(os.path.join(str(path_undistort),"checkerboard_3d.png"))
    return(triangulate)

def triangulatePoints( P1, P2, x1, x2 ):
    X = cv2.triangulatePoints( P1[:3], P2[:3], x1, x2 )
    return X/X[3]

    
def get_camerawise_videos(path,cam_names,videotype):
    """
    This function returns the list of videos corresponding to the camera names specified in the cam_names.
    e.g. if cam_names = ['camera-1','camera-2']
    
    then it will return [['somename-camera-1-othername.avi', 'somename-camera-2-othername.avi']]
    """
    import glob
    from pathlib import Path
    vid=[]
    
    # Find videos only specific to the cam names
    videos = [glob.glob(os.path.join(path,str('*'+cam_names[i]+'*'+videotype))) for i in range(len(cam_names)) ]
    videos = [y for x in videos for y in x]

    # Exclude the labeled video files
    if '.' in videotype:
        file_to_exclude = str('labeled'+videotype)
    else:
        file_to_exclude = str('labeled.'+videotype)
    videos=[v for v in videos if os.path.isfile(v) and not (file_to_exclude in v)]
    video_list=[]
    cam=cam_names[0] #camera1
    vid.append([name for name in glob.glob(os.path.join(path,str('*'+cam+'*'+videotype)))]) #all videos with cam 
    #print("here is what I found",vid)
    for k in range(len(vid[0])):
        if cam in str(Path(vid[0][k]).stem):
            ending=Path(vid[0][k]).suffix
            pref = str(Path(vid[0][k]).stem).split(cam)[0]
            suf = str(Path(vid[0][k]).stem).split(cam)[1]
            if pref == "":
                if suf=="":
                    print("Strange naming convention on your part. Respect.")
                else:
                    putativecam2name=os.path.join(path,cam_names[1]+suf+ending)
            else:
                if suf=="":
                    putativecam2name=os.path.join(path,pref+cam_names[1]+ending)
                else:
                    putativecam2name=os.path.join(path,pref+cam_names[1]+suf+ending)
            #print([os.path.join(path,pref+cam+suf+ending),putativecam2name])
            if os.path.isfile(putativecam2name): 
                #found a pair!!!
                video_list.append([os.path.join(path,pref+cam+suf+ending),putativecam2name])
    return(video_list)


def Get_list_of_triangulated_and_videoFiles(filepath,videotype,scorer_3d,cam_names,videofolder):
    """
    Returns the list of triangulated h5 and the corresponding video files.
    """

    prefix = []
    suffix=[]
    file_list = []
    string_to_search = scorer_3d+'.h5'

    # Checks if filepath is a directory
    if [os.path.isdir(i) for i in filepath] == [True]:
        """
        Analyzes all the videos in the directory.
        """
        print("Analyzing all the videos in the directory")
        videofolder= filepath[0]
        cwd = os.getcwd()
        os.chdir(videofolder)
        triangulated_file_list = [fn for fn in os.listdir(os.curdir) if (string_to_search in fn)]
        video_list = get_camerawise_videos(videofolder,cam_names,videotype)
        os.chdir(cwd)
        triangulated_folder = videofolder
    else:
        triangulated_file_list = [str(Path(fn).name) for fn in filepath if (string_to_search in fn)]
        triangulated_folder = [str(Path(fn).parents[0]) for fn in filepath if (string_to_search in fn)]
        triangulated_folder = triangulated_folder[0]

        if videofolder == None:
            videofolder = str(Path(filepath[0]).parents[0])
        video_list = get_camerawise_videos(videofolder,cam_names,videotype)

    # Get the filename of the triangulated file excluing the scorer name and remove any '-' or _ from it
    filename = [i.split(string_to_search)[0] for i in triangulated_file_list]
    for i in range(len(filename)):
        if filename[i][-1]=='_' or filename[i][-1]=='-':
            filename[i] = filename[i][:-1]
        if filename[i][0]=='_' or filename[i][0]=='-':
            filename[i] = filename[i][1:]

    # Get the suffix and prefix of the video filenames so that they can be used for matching the triangulated file names.
    for i in range(len(video_list)):
        pre = [str(Path(video_list[i][0]).stem).split(cam_names[0])[0],str(Path(video_list[i][1]).stem).split(cam_names[1])[0]]
        suf = [str(Path(video_list[i][0]).stem).split(cam_names[0])[-1],str(Path(video_list[i][1]).stem).split(cam_names[1])[-1]]
        for i in range(len(cam_names)):
            if pre[i] == "":
                pass
            elif pre[i][-1]=='_' or pre[i][-1]=='-':
                pre[i] = pre[i][:-1]
            if suf[i]=="":
                pass
            elif suf[i][0]=='_' or suf[i][0]=='-':
                suf[i] = suf[i][1:]
        suffix.append(suf)
        prefix.append(pre)

    # Match the suffix and prefix with the triangulated file name and return the list with triangulated file and corresponding video files.
    for k in range(len(filename)):
        for j in range(len(prefix)):
            if (prefix[j][0] in filename[k] and prefix[j][1] in filename[k]) and (suffix[j][0] in filename[k] and suffix[j][1] in filename[k]):
                triangulated_file = glob.glob(os.path.join(triangulated_folder,str('*'+filename[k]+'*'+string_to_search)))
                vfiles = get_camerawise_videos(videofolder,cam_names,videotype)
                vfiles = [z for z in vfiles if prefix[j][0] in z[0] and suffix[j][0] in z[1]][0]
                file_list.append(triangulated_file+vfiles)

    return file_list


def SaveMetadata3d(metadatafilename, metadata):
        with open(metadatafilename, 'wb') as f:
            pickle.dump(metadata, f,pickle.HIGHEST_PROTOCOL)
            
def LoadMetadata3d(metadatafilename):
    with open(metadatafilename, 'rb') as f:
        metadata= pickle.load(f)
        return metadata


"""
Adapted from DeeperCut by Eldar Insafutdinov:
https://github.com/eldar/pose-tensorflow
"""

import os
from deeplabcut.pose_estimation_tensorflow.util.logging import *

if os.environ.get('DLClight', default=False) == 'True':
    pass
else:
    from deeplabcut.pose_estimation_tensorflow.util.visualize import *



"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut

Please see AUTHORS for contributors.
https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0
"""

import pandas as pd
import numpy as np
from pathlib import Path
import cv2
import os
import glob
import subprocess
import matplotlib.pyplot as plt
from deeplabcut.utils import auxiliaryfunctions
from deeplabcut.utils import auxiliaryfunctions_3d
from matplotlib.axes._axes import _log as matplotlib_axes_logger
matplotlib_axes_logger.setLevel('ERROR')
from matplotlib import gridspec
from tqdm import tqdm

def create_labeled_video_3d(config,path,videofolder=None,start=0,end=None,trailpoints=0,videotype='avi',view=[-113, -270],xlim=[None,None],ylim=[None,None],zlim=[None,None],draw_skeleton=True):
    """
    Creates a video with views from the two cameras and the 3d reconstruction for a selected number of frames.

    Parameters
    ----------
    config : string
        Full path of the config.yaml file as a string.

    path : list
        A list of strings containing the full paths to triangulated files for analysis or a path to the directory, where all the triangulated files are stored.

    videofolder: string
        Full path of the folder where the videos are stored. Use this if the vidoes are stored in a different location other than where the triangulation files are stored. By default is ``None`` and therefore looks for video files in the directory where the triangulation file is stored.

    start: int
        Integer specifying the start of frame index to select. Default is set to 0.

    end: int
        Integer specifying the end of frame index to select. Default is set to None, where all the frames of the video are used for creating the labeled video.

    trailpoints: int
        Number of revious frames whose body parts are plotted in a frame (for displaying history). Default is set to 0.

    videotype: string, optional
        Checks for the extension of the video in case the input is a directory.\nOnly videos with this extension are analyzed. The default is ``.avi``

    view: list
        A list that sets the elevation angle in z plane and azimuthal angle in x,y plane of 3d view. Useful for rotating the axis for 3d view

    xlim: list
        A list of integers specifying the limits for xaxis of 3d view. By default it is set to [None,None], where the x limit is set by taking the minimum and maximum value of the x coordinates for all the bodyparts.

    ylim: list
        A list of integers specifying the limits for yaxis of 3d view. By default it is set to [None,None], where the y limit is set by taking the minimum and maximum value of the y coordinates for all the bodyparts.

    zlim: list
        A list of integers specifying the limits for zaxis of 3d view. By default it is set to [None,None], where the z limit is set by taking the minimum and maximum value of the z coordinates for all the bodyparts.

    draw_skeleton: bool
        If ``True`` adds a line connecting the body parts making a skeleton on on each frame. The body parts to be connected and the color of these connecting lines are specified in the config file. By default: ``True``

    Example
    -------
    Linux/MacOs
    >>> deeplabcut.create_labeled_video_3d(config,['/data/project1/videos/3d.h5'],start=100, end=500)

    To create labeled videos for all the triangulated files in the folder
    >>> deeplabcut.create_labeled_video_3d(config,['/data/project1/videos'],start=100, end=500)

    To set the xlim, ylim, zlim and rotate the view of the 3d axis
    >>> deeplabcut.create_labeled_video_3d(config,['/data/project1/videos'],start=100, end=500,view=[30,90],xlim=[-12,12],ylim=[15,25],zlim=[20,30])

    """
    start_path=os.getcwd()

    # Read the config file and related variables
    cfg_3d = auxiliaryfunctions.read_config(config)
    cam_names = cfg_3d['camera_names']
    pcutoff = cfg_3d['pcutoff']
    markerSize = cfg_3d['dotsize']
    alphaValue = cfg_3d['alphaValue']
    cmap = cfg_3d['colormap']
    bodyparts2connect = cfg_3d['skeleton']
    skeleton_color = cfg_3d['skeleton_color']
    scorer_3d = cfg_3d['scorername_3d']

    # Flatten the list of bodyparts to connect
    bodyparts2plot = list(np.unique([val for sublist in bodyparts2connect for val in sublist]))
    color = plt.cm.get_cmap(cmap, len(bodyparts2plot))
    file_list = auxiliaryfunctions_3d.Get_list_of_triangulated_and_videoFiles(path,videotype,scorer_3d,cam_names,videofolder)
    print(file_list)
    if file_list == []:
        raise Exception("No corresponding video file(s) found for the specified triangulated file or folder. Did you specify the video file type? If videos are stored in a different location, please use the ``videofolder`` argument to specify their path.")

    for file in file_list:
        path_h5_file = Path(file[0]).parents[0]
        triangulate_file = file[0]
        # triangulated file is a list which is always sorted as [triangulated.h5,camera-1.videotype,camera-2.videotype]
        #name for output video
        file_name = str(Path(triangulate_file).stem)
        if os.path.isfile(os.path.join(path_h5_file,file_name+'.mpg')):
            print("Video already created...")
        else:
            string_to_remove = str(Path(triangulate_file).suffix)
            pickle_file = triangulate_file.replace(string_to_remove,'_includingmetadata.pickle')
            metadata_ = auxiliaryfunctions_3d.LoadMetadata3d(pickle_file)

            base_filename_cam1 = str(Path(file[1]).stem).split(videotype)[0] # required for searching the filtered file
            base_filename_cam2 = str(Path(file[2]).stem).split(videotype)[0] # required for searching the filtered file
            cam1_view_video = file[1]
            cam2_view_video = file[2]
            cam1_scorer = metadata_['scorer_name'][cam_names[0]]
            cam2_scorer = metadata_['scorer_name'][cam_names[1]]
            print("Creating 3D video from %s and %s using %s"%(Path(cam1_view_video).name,Path(cam2_view_video).name,Path(triangulate_file).name))

            # Read the video files and corresponfing h5 files
            vid_cam1 = cv2.VideoCapture(cam1_view_video)
            vid_cam2 = cv2.VideoCapture(cam2_view_video)

            # Look for the filtered predictions file
            try:
                print("Looking for filtered predictions...")
                df_cam1= pd.read_hdf(glob.glob(os.path.join(path_h5_file,str('*'+base_filename_cam1+cam1_scorer+'*filtered.h5')))[0])
                df_cam2 = pd.read_hdf(glob.glob(os.path.join(path_h5_file,str('*'+base_filename_cam2+cam2_scorer+'*filtered.h5')))[0])
                #print("Found filtered predictions, will be use these for triangulation.")
                print("Found the following filtered data: ",os.path.join(path_h5_file,str('*'+base_filename_cam1+cam1_scorer+'*filtered.h5')),os.path.join(path_h5_file,str('*'+base_filename_cam2+cam2_scorer+'*filtered.h5')))
            except FileNotFoundError:
                print("No filtered predictions found, the unfiltered predictions will be used instead.")
                df_cam1= pd.read_hdf(glob.glob(os.path.join(path_h5_file,str(base_filename_cam1+cam1_scorer+'*.h5')))[0])
                df_cam2 = pd.read_hdf(glob.glob(os.path.join(path_h5_file,str(base_filename_cam2+cam2_scorer+'*.h5')))[0])

            df_3d = pd.read_hdf(triangulate_file,'df_with_missing')
            plt.rcParams.update({'figure.max_open_warning': 0})

            if end==None:
                end=len(df_3d) # All the frames
            frames = list(range(start,end,1))

            # Start plotting for every frame
            for k in tqdm(frames):
                output_folder,num_frames = plot2D(cfg_3d,k,bodyparts2plot,vid_cam1,vid_cam2,bodyparts2connect,df_cam1,df_cam2,df_3d,pcutoff,markerSize,alphaValue,color,path_h5_file,file_name,skeleton_color,view,draw_skeleton,trailpoints,xlim,ylim,zlim)

            # Once all the frames are saved, then make a movie using ffmpeg.
            cwd = os.getcwd()
            os.chdir(str(output_folder))
            subprocess.call(['ffmpeg', '-start_number', str(start) ,'-framerate', str(30), '-i', str('img%0'+str(num_frames)+'d.png'), '-r', str(30),'-vb', '20M' ,os.path.join(output_folder,str('../'+file_name+'.mpg'))])
            os.chdir(cwd)

    os.chdir(start_path)

def plot2D(cfg_3d,k,bodyparts2plot,vid_cam1,vid_cam2,bodyparts2connect,df_cam1_view,df_cam2_view,xyz_pts,pcutoff,markerSize,alphaValue,color,path_h5_file,file_name,skeleton_color,view,draw_skeleton,trailpoints,xlim,ylim,zlim):
    """
    Creates 2D gif for a selected number of frames
    """
    # Create the fig and define the axes
    gs = gridspec.GridSpec(1, 3,width_ratios=[1,1,1])
    fig = plt.figure(figsize=(20,8))
    axes1 = fig.add_subplot(gs[0, 0]) # row 0, col 0
    axes2 = fig.add_subplot(gs[0, 1]) # row 0, col 1
    axes3 = fig.add_subplot(gs[0, 2],projection='3d') # row 1, span all columns
    fig.tight_layout()

    # Clear plot and initialize the variables
    plt.cla()
    axes1.cla()
    axes2.cla()
    axes3.cla()

    # Initialize arrays for appending the 3d data for actual plotting the points
    xdata_3d = []
    ydata_3d = []
    zdata_3d = []

    # Initialize arrays for appending the 3d data for drawing the lines
    xlines_3d = []
    ylines_3d = []
    zlines_3d = []

    # Initialize arrays for appending the 2d data from cam1 for actual plotting the points
    xdata_cam1=[]
    ydata_cam1=[]

    # Initialize arrays for appending the 2d data from cam2 for actual plotting the points
    xdata_cam2=[]
    ydata_cam2=[]

    # Initialize arrays for appending the 2d data from cam1 for drawing the lines
    xcam1 = []
    ycam1 = []

    # Initialize arrays for appending the 2d data from cam2 for drawing the lines
    xcam2 = []
    ycam2 = []

    # Get the scorer names from the dataframe
    scorer_cam1 = df_cam1_view.columns.get_level_values(0)[0]
    scorer_cam2 = df_cam2_view.columns.get_level_values(0)[0]
    scorer_3d = xyz_pts.columns.get_level_values(0)[0]

    # Set the x,y, and z limits for the 3d view

    numberFrames = min([int(vid_cam1.get(cv2.CAP_PROP_FRAME_COUNT)),int(vid_cam2.get(cv2.CAP_PROP_FRAME_COUNT))]) #minimum of two cameras / TODO: clean up!
    df_x = np.empty((len(bodyparts2plot),numberFrames))
    df_y = np.empty((len(bodyparts2plot),numberFrames))
    df_z = np.empty((len(bodyparts2plot),numberFrames))
    for bpindex, bp in enumerate(bodyparts2plot):
        df_x[bpindex,:]=xyz_pts[scorer_3d][bp]['x'].values
        df_y[bpindex,:]=xyz_pts[scorer_3d][bp]['y'].values
        df_z[bpindex,:]=xyz_pts[scorer_3d][bp]['z'].values
    if xlim==[None,None]:
        axes3.set_xlim3d([np.nanmin(df_x),np.nanmax(df_x)])
    else:
        axes3.set_xlim3d(xlim)
    if ylim==[None,None]:
        axes3.set_ylim3d([np.nanmin(df_y),np.nanmax(df_y)])
    else:
        axes3.set_ylim3d(ylim)
    if zlim==[None,None]:
        axes3.set_zlim3d([np.nanmin(df_z),np.nanmax(df_z)])
    else:
        axes3.set_zlim3d(zlim)

    axes3.set_xticklabels([])
    axes3.set_yticklabels([])
    axes3.set_zticklabels([])
    axes3.xaxis.grid(False)
    axes3.view_init(view[0], view[1])
    axes3.set_xlabel('X', fontsize=10)
    axes3.set_ylabel('Y', fontsize=10)
    axes3.set_zlabel('Z', fontsize=10)


    # Set the frame number to read#max(0,index-trailpoints):index
    vid_cam1.set(1,k)
    vid_cam2.set(1,k)
    ret_cam1, frame_cam1 = vid_cam1.read() #TODO: use ret_camj
    ret_cam2,frame_cam2 = vid_cam2.read()

    # Plot the labels for each body part
    for bpindex,bp in enumerate(bodyparts2plot):
        axes1.imshow(cv2.cvtColor(frame_cam1, cv2.COLOR_BGR2RGB))
        axes2.imshow(cv2.cvtColor(frame_cam2, cv2.COLOR_BGR2RGB))
        if (df_cam1_view[scorer_cam1][bp]['likelihood'].values[k]) > pcutoff and (df_cam2_view[scorer_cam2][bp]['likelihood'].values[k]) > pcutoff:
            if trailpoints > 0:
                xdata_cam1.append(df_cam1_view.iloc[max(0,k-trailpoints):k][scorer_cam1][bp]['x'])
                ydata_cam1.append(df_cam1_view.iloc[max(0,k-trailpoints):k][scorer_cam1][bp]['y'])
                xdata_cam2.append(df_cam2_view.iloc[max(0,k-trailpoints):k][scorer_cam2][bp]['x'])
                ydata_cam2.append(df_cam2_view.iloc[max(0,k-trailpoints):k][scorer_cam2][bp]['y'])
                xdata_3d.append(xyz_pts.iloc[max(0,k-trailpoints):k][scorer_3d][bp]['x'])
                ydata_3d.append(xyz_pts.iloc[max(0,k-trailpoints):k][scorer_3d][bp]['y'])
                zdata_3d.append(xyz_pts.iloc[max(0,k-trailpoints):k][scorer_3d][bp]['z'])
            else:
                xdata_cam1.append(df_cam1_view.iloc[k][scorer_cam1][bp]['x'])
                ydata_cam1.append(df_cam1_view.iloc[k][scorer_cam1][bp]['y'])
                xdata_cam2.append(df_cam2_view.iloc[k][scorer_cam2][bp]['x'])
                ydata_cam2.append(df_cam2_view.iloc[k][scorer_cam2][bp]['y'])
                xdata_3d.append(xyz_pts.iloc[k][scorer_3d][bp]['x'])
                ydata_3d.append(xyz_pts.iloc[k][scorer_3d][bp]['y'])
                zdata_3d.append(xyz_pts.iloc[k][scorer_3d][bp]['z'])
        else:
            xdata_cam1.append(np.nan)
            ydata_cam1.append(np.nan)
            xdata_cam2.append(np.nan)
            ydata_cam2.append(np.nan)
            xdata_3d.append(np.nan)
            ydata_3d.append(np.nan)
            zdata_3d.append(np.nan)
        p =axes1.scatter(xdata_cam1[bpindex],ydata_cam1[bpindex],s=markerSize,c=color(bodyparts2plot.index(bp)))
        p =axes2.scatter(xdata_cam2[bpindex],ydata_cam2[bpindex],s=markerSize,c=color(bodyparts2plot.index(bp)))
        p =axes3.scatter(xdata_3d[bpindex],ydata_3d[bpindex],zdata_3d[bpindex],c=color(bodyparts2plot.index(bp)))

    # Connecting the bodyparts specified in the config file.3d file is created based on the likelihoods of cam1 and cam2. Using 3d file and check if the body part is nan then dont plot skeleton
    if draw_skeleton:
        for i in range(len(bodyparts2connect)):
            bool_above_pcutoff = [np.isnan(xyz_pts.iloc[k][scorer_3d][bodyparts2connect[i][0]]['x']) or (np.isnan(xyz_pts.iloc[k][scorer_3d][bodyparts2connect[i][1]]['x']))]
            if not bool_above_pcutoff[0]:
                xcam1.append(df_cam1_view.iloc[k][scorer_cam1][bodyparts2connect[i][0]]['x'])
                ycam1.append(df_cam1_view.iloc[k][scorer_cam1][bodyparts2connect[i][0]]['y'])
                xcam1.append(df_cam1_view.iloc[k][scorer_cam1][bodyparts2connect[i][1]]['x'])
                ycam1.append(df_cam1_view.iloc[k][scorer_cam1][bodyparts2connect[i][1]]['y'])

                xcam2.append(df_cam2_view.iloc[k][scorer_cam2][bodyparts2connect[i][0]]['x'])
                ycam2.append(df_cam2_view.iloc[k][scorer_cam2][bodyparts2connect[i][0]]['y'])
                xcam2.append(df_cam2_view.iloc[k][scorer_cam2][bodyparts2connect[i][1]]['x'])
                ycam2.append(df_cam2_view.iloc[k][scorer_cam2][bodyparts2connect[i][1]]['y'])

                xlines_3d.append(xyz_pts.iloc[k][scorer_3d][bodyparts2connect[i][0]]['x'])
                ylines_3d.append(xyz_pts.iloc[k][scorer_3d][bodyparts2connect[i][0]]['y'])
                zlines_3d.append(xyz_pts.iloc[k][scorer_3d][bodyparts2connect[i][0]]['z'])
                xlines_3d.append(xyz_pts.iloc[k][scorer_3d][bodyparts2connect[i][1]]['x'])
                ylines_3d.append(xyz_pts.iloc[k][scorer_3d][bodyparts2connect[i][1]]['y'])
                zlines_3d.append(xyz_pts.iloc[k][scorer_3d][bodyparts2connect[i][1]]['z'])

                axes1.plot(xcam1,ycam1, color=skeleton_color,alpha=alphaValue)
                axes2.plot(xcam2,ycam2, color=skeleton_color,alpha=alphaValue)
                axes3.plot(xlines_3d,ylines_3d,zlines_3d, color=skeleton_color,alpha=alphaValue)

                xcam1 = []
                ycam1 = []
                xcam2 = []
                ycam2 = []
                xlines_3d = []
                ylines_3d = []
                zlines_3d = []
    # Saving the frames
    output_folder = Path(os.path.join(path_h5_file,'temp_'+file_name))
    output_folder.mkdir(parents = True, exist_ok = True)
    num_frames = int(np.ceil(np.log10(numberFrames)))
    img_name = str(output_folder) +'/img'+str(k).zfill(num_frames) + '.png'
    plt.savefig(img_name)
    plt.close('all')
    return(output_folder,num_frames)




"""
DeepLabCut2.0 Toolbox (deeplabcut.org)
© A. & M. Mathis Labs
https://github.com/AlexEMG/DeepLabCut
Please see AUTHORS for contributors.

https://github.com/AlexEMG/DeepLabCut/blob/master/AUTHORS
Licensed under GNU Lesser General Public License v3.0

"""

import wx
import os,sys,pydoc
import deeplabcut
media_path = os.path.join(deeplabcut.__path__[0], 'gui' , 'media')
logo = os.path.join(media_path,'logo.png')

from deeplabcut.utils import auxiliaryfunctions

class Create_Labeled_Videos(wx.Panel):
    """
    """

    def __init__(self, parent,gui_size,cfg):
        """Constructor"""
        wx.Panel.__init__(self, parent=parent)
        # variable initilization
        self.filelist = []
        self.config = cfg
        self.bodyparts = []
        self.draw = False
        self.slow = False
        # design the panel
        self.sizer = wx.GridBagSizer(5, 5)

        text = wx.StaticText(self, label="DeepLabCut - Create Labeled Videos (more functionality)")
        self.sizer.Add(text, pos=(0, 0), flag=wx.TOP|wx.LEFT|wx.BOTTOM,border=15)
        # Add logo of DLC
        icon = wx.StaticBitmap(self, bitmap=wx.Bitmap(logo))
        self.sizer.Add(icon, pos=(0, 4), flag=wx.TOP|wx.RIGHT|wx.ALIGN_RIGHT,border=5)

        line1 = wx.StaticLine(self)
        self.sizer.Add(line1, pos=(1, 0), span=(1, 5),flag=wx.EXPAND|wx.BOTTOM, border=10)

        self.cfg_text = wx.StaticText(self, label="Select the config file")
        self.sizer.Add(self.cfg_text, pos=(2, 0), flag=wx.TOP|wx.LEFT, border=5)

        if sys.platform=='darwin':
            self.sel_config = wx.FilePickerCtrl(self, path="",style=wx.FLP_USE_TEXTCTRL,message="Choose the config.yaml file", wildcard="*.yaml")
        else:
            self.sel_config = wx.FilePickerCtrl(self, path="",style=wx.FLP_USE_TEXTCTRL,message="Choose the config.yaml file", wildcard="config.yaml")

        self.sizer.Add(self.sel_config, pos=(2, 1),span=(1,3),flag=wx.TOP|wx.EXPAND, border=5)
        self.sel_config.SetPath(self.config)
        self.sel_config.Bind(wx.EVT_FILEPICKER_CHANGED, self.select_config)

        self.vids = wx.StaticText(self, label="Choose the videos")
        self.sizer.Add(self.vids, pos=(3, 0), flag=wx.TOP|wx.LEFT, border=10)

        self.sel_vids = wx.Button(self, label="Select videos")
        self.sizer.Add(self.sel_vids, pos=(3, 1), flag=wx.TOP|wx.EXPAND, border=5)
        self.sel_vids.Bind(wx.EVT_BUTTON, self.select_videos)

        sb = wx.StaticBox(self, label="Additional Attributes")
        boxsizer = wx.StaticBoxSizer(sb, wx.VERTICAL)

        hbox1 = wx.BoxSizer(wx.HORIZONTAL)
        hbox2 = wx.BoxSizer(wx.HORIZONTAL)
        hbox3 = wx.BoxSizer(wx.HORIZONTAL)

        videotype_text = wx.StaticBox(self, label="Specify the videotype")
        videotype_text_boxsizer = wx.StaticBoxSizer(videotype_text, wx.VERTICAL)

        videotypes = ['.avi', '.mp4', '.mov']
        self.videotype = wx.ComboBox(self,choices = videotypes,style = wx.CB_READONLY)
        self.videotype.SetValue('.avi')
        videotype_text_boxsizer.Add(self.videotype,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        shuffle_text = wx.StaticBox(self, label="Specify the shuffle")
        shuffle_boxsizer = wx.StaticBoxSizer(shuffle_text, wx.VERTICAL)
        self.shuffle = wx.SpinCtrl(self, value='1',min=1,max=100)
        shuffle_boxsizer.Add(self.shuffle,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        trainingset = wx.StaticBox(self, label="Specify the trainingset index")
        trainingset_boxsizer = wx.StaticBoxSizer(trainingset, wx.VERTICAL)
        self.trainingset = wx.SpinCtrl(self, value='0',min=0,max=100)
        trainingset_boxsizer.Add(self.trainingset,1, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        hbox1.Add(videotype_text_boxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox1.Add(shuffle_boxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        hbox1.Add(trainingset_boxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 5)
        boxsizer.Add(hbox1,0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        self.sizer.Add(boxsizer, pos=(4, 0), span=(1, 5),flag=wx.EXPAND|wx.TOP|wx.LEFT|wx.RIGHT , border=10)

        self.draw_skeleton = wx.RadioBox(self, label='Include the skeleton in the video?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.draw_skeleton.Bind(wx.EVT_RADIOBOX, self.choose_draw_skeleton_options)
        self.draw_skeleton.SetSelection(0)

        self.filter = wx.RadioBox(self, label='Use filtered predictions?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.filter.SetSelection(1)

        self.video_slow = wx.RadioBox(self, label='Create a higher quality video? (slow)', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.video_slow.Bind(wx.EVT_RADIOBOX, self.choose_video_slow_options)
        self.video_slow.SetSelection(1)


        self.trail_points_text = wx.StaticBox(self, label="Specify the number of trail points")
        trail_pointsboxsizer = wx.StaticBoxSizer(self.trail_points_text, wx.VERTICAL)
        self.trail_points = wx.SpinCtrl(self, value='0')
        trail_pointsboxsizer.Add(self.trail_points,20, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)


        self.bodypart_choice = wx.RadioBox(self, label='Plot all bodyparts?', choices=['Yes', 'No'],majorDimension=1, style=wx.RA_SPECIFY_COLS)
        self.bodypart_choice.Bind(wx.EVT_RADIOBOX,self.chooseOption)

        config_file = auxiliaryfunctions.read_config(self.config)
        bodyparts = config_file['bodyparts']
        self.bodyparts_to_compare = wx.CheckListBox(self, choices=bodyparts, style=0,name = "Select the bodyparts")
        self.bodyparts_to_compare.Bind(wx.EVT_CHECKLISTBOX,self.getbp)
        self.bodyparts_to_compare.Hide()

        hbox2.Add(self.draw_skeleton,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        hbox2.Add(trail_pointsboxsizer,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        hbox2.Add(self.video_slow, 10, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        boxsizer.Add(hbox2,0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        hbox3.Add(self.filter,10,wx.EXPAND|wx.TOP|wx.BOTTOM,10)
        hbox3.Add(self.bodypart_choice,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        hbox3.Add(self.bodyparts_to_compare,10, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)
        boxsizer.Add(hbox3,0, wx.EXPAND|wx.TOP|wx.BOTTOM, 10)

        self.help_button = wx.Button(self, label='Help')
        self.sizer.Add(self.help_button, pos=(5, 0), flag=wx.LEFT, border=10)
        self.help_button.Bind(wx.EVT_BUTTON, self.help_function)

        self.ok = wx.Button(self, label="RUN")
        self.sizer.Add(self.ok, pos=(5, 4))
        self.ok.Bind(wx.EVT_BUTTON, self.create_videos)

        self.reset = wx.Button(self, label="Reset")
        self.sizer.Add(self.reset, pos=(5, 1), span=(1, 1),flag=wx.BOTTOM|wx.RIGHT, border=10)
        self.reset.Bind(wx.EVT_BUTTON, self.reset_create_videos)

        self.sizer.AddGrowableCol(3)

        self.SetSizer(self.sizer)
        self.sizer.Fit(self)

    def select_config(self,event):
        """
        """
        self.config = self.sel_config.GetPath()

    def select_videos(self,event):
        """
        Selects the videos from the directory
        """
        cwd = os.getcwd()
        dlg = wx.FileDialog(self, "Select videos", cwd, "", "*.*", wx.FD_MULTIPLE)
        if dlg.ShowModal() == wx.ID_OK:
            self.vids = dlg.GetPaths()
            self.filelist = self.filelist + self.vids
            self.sel_vids.SetLabel("Total %s Videos selected" %len(self.filelist))

    def choose_draw_skeleton_options(self,event):
        if self.draw_skeleton.GetStringSelection() == "Yes":
            self.draw = True
        else:
            self.draw = False

    def choose_video_slow_options(self,event):
        if self.video_slow.GetStringSelection() == "Yes":
            self.slow = True
        else:
            self.slow = False

    def chooseOption(self,event):
        if self.bodypart_choice.GetStringSelection() == 'No':
            self.bodyparts_to_compare.Show()
            self.getbp(event)
            self.SetSizer(self.sizer)
            self.sizer.Fit(self) #this sets location.
        if self.bodypart_choice.GetStringSelection() == 'Yes':
            self.bodyparts_to_compare.Hide()
            self.SetSizer(self.sizer)
            self.sizer.Fit(self)
            self.bodyparts = 'all'

    def getbp(self,event):
        self.bodyparts = list(self.bodyparts_to_compare.GetCheckedStrings())

    def create_videos(self,event):

        shuffle = self.shuffle.GetValue()
        trainingsetindex = self.trainingset.GetValue()

        if self.filter.GetStringSelection() == "No":
            filter = None
        else:
            filter = True

        if self.video_slow.GetStringSelection() == "Yes":
            self.slow = True
        else:
            self.slow = False

        if self.filter.GetStringSelection() == "Yes":
            if len(self.bodyparts)==0:
                self.bodyparts='all'

                deeplabcut.create_labeled_video(self.config,self.filelist,self.videotype.GetValue(),shuffle=shuffle, trainingsetindex=trainingsetindex, save_frames=self.slow, draw_skeleton= self.draw, displayedbodyparts=self.bodyparts, trailpoints = self.trail_points.GetValue(), filtered=True)

        if len(self.bodyparts)==0:
            self.bodyparts='all'
        deeplabcut.create_labeled_video(self.config,self.filelist,self.videotype.GetValue(),shuffle=shuffle, trainingsetindex=trainingsetindex, save_frames=self.slow, draw_skeleton= self.draw, displayedbodyparts=self.bodyparts, trailpoints = self.trail_points.GetValue(), filtered=False)



    def help_function(self,event):

        filepath= 'help.txt'
        f = open(filepath, 'w')
        sys.stdout = f
        fnc_name = 'deeplabcut.create_labeled_video'
        pydoc.help(fnc_name)
        f.close()
        sys.stdout = sys.__stdout__
        help_file = open("help.txt","r+")
        help_text = help_file.read()
        wx.MessageBox(help_text,'Help',wx.OK | wx.ICON_INFORMATION)
        os.remove('help.txt')




    def reset_create_videos(self,event):
        """
        Reset to default
        """
        self.config = []
        self.sel_config.SetPath("")
        self.videotype.SetStringSelection(".avi")
        self.sel_vids.SetLabel("Select videos")
        self.filelist = []
        self.shuffle.SetValue(1)
        self.trainingset.SetValue(0)
        if self.draw_skeleton.IsShown():
            self.draw_skeleton.SetSelection(1)
            #self.SetSizer(self.sizer)
            #self.sizer.Fit(self)
            self.bodyparts_to_compare.Hide()
